@@ -2,7 +2,7 @@ from .region_identifier import RegionIdentifier
 from .structurer import Structurer
 from .structured_codegen import CStructuredCodeGenerator, ImportSourceCode
 from .clinic import Clinic
-from .region_simplifier import RegionSimplifier
+from .region_simplifiers import RegionSimplifier
 from .decompiler import Decompiler
 from .decompilation_options import options, options_by_category
 from .block_simplifier import BlockSimplifier
