@@ -88,14 +88,8 @@ class CallSiteMaker(Analysis):
                     if type(arg_loc) is SimRegArg:
                         size = arg_loc.size
                         offset = arg_loc.check_offset(cc.arch)
-
-                        _, the_arg = self._resolve_register_argument(last_stmt, arg_loc)
-
-                        if the_arg is not None:
-                            args.append(the_arg)
-                        else:
-                            # Reaching definitions are not available. Create a register expression instead.
-                            args.append(Expr.Register(self._atom_idx(), None, offset, size * 8, reg_name=arg_loc.reg_name))
+                        args.append(Expr.Register(self._atom_idx(), None, offset, size * 8,
+                                                  reg_name=arg_loc.reg_name))
                     elif type(arg_loc) is SimStackArg:
 
                         stack_arg_locs.append(arg_loc)
@@ -157,7 +151,17 @@ class CallSiteMaker(Analysis):
         # if ret_expr is None, it means in previous steps (such as during AIL simplification) we have deemed the return
         # value of this call statement as useless and is removed.
 
-        new_stmts.append(Stmt.Call(last_stmt, last_stmt.target,
+        if ret_expr is not None \
+                and prototype is not None \
+                and prototype.returnty is not None \
+                and not isinstance(prototype.returnty, SimTypeBottom):
+            # try to narrow the return expression if needed
+            ret_type_bits = prototype.returnty.with_arch(self.project.arch).size
+            if ret_expr.bits > ret_type_bits:
+                ret_expr = ret_expr.copy()
+                ret_expr.bits = ret_type_bits
+
+        new_stmts.append(Stmt.Call(last_stmt.idx, last_stmt.target,
                                    calling_convention=cc,
                                    prototype=prototype,
                                    args=args,
