@@ -1,12 +1,14 @@
+import copy
 from collections import defaultdict
 import logging
-from typing import Dict, List, Tuple, Set, Optional, Iterable, Union, Type, TYPE_CHECKING
+from typing import Dict, List, Tuple, Set, Optional, Iterable, Union, Type, Any, TYPE_CHECKING
 
 import networkx
 
 import ailment
 
 from ...knowledge_base import KnowledgeBase
+from ...knowledge_plugins.functions import Function
 from ...codenode import BlockNode
 from ...utils import timethis
 from ...calling_conventions import SimRegArg, SimStackArg, SimFunctionArgument
@@ -17,6 +19,7 @@ from ...procedures.stubs.UnresolvableCallTarget import UnresolvableCallTarget
 from ...procedures.stubs.UnresolvableJumpTarget import UnresolvableJumpTarget
 from .. import Analysis, register_analysis
 from ..cfg.cfg_base import CFGBase
+from ..reaching_definitions import ReachingDefinitionsAnalysis
 from .ailgraph_walker import AILGraphWalker
 from .ailblock_walker import AILBlockWalker
 from .optimization_passes import get_default_optimization_passes, OptimizationPassStage
@@ -26,7 +29,6 @@ if TYPE_CHECKING:
     from .decompilation_cache import DecompilationCache
     from .peephole_optimizations import PeepholeOptimizationStmtBase, PeepholeOptimizationExprBase
 
-
 l = logging.getLogger(name=__name__)
 
 
@@ -34,13 +36,14 @@ class Clinic(Analysis):
     """
     A Clinic deals with AILments.
     """
+
     def __init__(self, func,
                  remove_dead_memdefs=False,
                  exception_edges=False,
                  sp_tracker_track_memory=True,
                  optimization_passes=None,
                  cfg=None,
-                 peephole_optimizations: Optional[Iterable[Union[Type['PeepholeOptimizationStmtBase'],Type['PeepholeOptimizationExprBase']]]]=None,  # pylint:disable=line-too-long
+                 peephole_optimizations: Optional[Iterable[Union[Type['PeepholeOptimizationStmtBase'],Type['PeepholeOptimizationExprBase']]]]=None, # pylint:disable=line-too-long
                  must_struct: Optional[Set[str]]=None,
                  variable_kb=None,
                  reset_variable_names=False,
@@ -52,12 +55,14 @@ class Clinic(Analysis):
         self.function = func
 
         self.graph = None
+        self.cc_graph: Optional[networkx.DiGraph] = None
         self.arg_list = None
         self.variable_kb = variable_kb
+        self.externs: Set[SimMemoryVariable] = set()
 
         self._func_graph: Optional[networkx.DiGraph] = None
         self._ail_manager = None
-        self._blocks_by_addr_and_size = { }
+        self._blocks_by_addr_and_size = {}
 
         self._remove_dead_memdefs = remove_dead_memdefs
         self._exception_edges = exception_edges
@@ -66,6 +71,7 @@ class Clinic(Analysis):
         self.peephole_optimizations = peephole_optimizations
         self._must_struct = must_struct
         self._reset_variable_names = reset_variable_names
+        self.reaching_definitions: Optional[ReachingDefinitionsAnalysis] = None
         self._cache = cache
 
         # sanity checks
@@ -129,7 +135,7 @@ class Clinic(Analysis):
         if not self._func_graph:
             return
 
-        # Make sure calling conventions of all functions have been recovered
+        # Make sure calling conventions of all functions that the current function calls have been recovered
         self._update_progress(10., text="Recovering calling conventions")
         self._recover_calling_conventions()
 
@@ -195,6 +201,10 @@ class Clinic(Analysis):
         self._simplify_function(ail_graph, remove_dead_memdefs=self._remove_dead_memdefs,
                                 stack_arg_offsets=stackarg_offsets, unify_variables=True)
 
+        self._update_progress(68., text="Simplifying blocks 3")
+        ail_graph = self._simplify_blocks(ail_graph, remove_dead_memdefs=self._remove_dead_memdefs,
+                                          stack_pointer_tracker=spt)
+
         # Make function arguments
         self._update_progress(70., text="Making argument list")
         arg_list = self._make_argument_list()
@@ -213,11 +223,37 @@ class Clinic(Analysis):
 
         # Run simplification passes
         self._update_progress(90., text="Running simplifications 3")
-        ail_graph = self._run_simplification_passes(ail_graph, stage=OptimizationPassStage.AFTER_VARIABLE_RECOVERY)
+        ail_graph = self._run_simplification_passes(ail_graph, stage=OptimizationPassStage.AFTER_VARIABLE_RECOVERY,
+                                                    variable_kb=variable_kb)
 
         self.graph = ail_graph
         self.arg_list = arg_list
         self.variable_kb = variable_kb
+        self.cc_graph = self.copy_graph()
+        self.externs = self._collect_externs(ail_graph, variable_kb)
+
+    def copy_graph(self) -> networkx.DiGraph:
+        """
+        Copy AIL Graph.
+
+        :return: A copy of the AIl graph.
+        """
+        graph_copy = networkx.DiGraph()
+        block_mapping = { }
+        # copy all blocks
+        for block in self.graph.nodes():
+            new_block = copy.copy(block)
+            new_stmts = copy.copy(block.statements)
+            new_block.statements = new_stmts
+            block_mapping[block] = new_block
+            graph_copy.add_node(new_block)
+
+        # copy all edges
+        for src, dst, data in self.graph.edges(data=True):
+            new_src = block_mapping[src]
+            new_dst = block_mapping[dst]
+            graph_copy.add_edge(new_src, new_dst, **data)
+        return graph_copy
 
     @timethis
     def _set_function_graph(self):
@@ -235,7 +271,19 @@ class Clinic(Analysis):
 
     @timethis
     def _recover_calling_conventions(self):
-        self.project.analyses.CompleteCallingConventions()
+
+        callees = set()
+        for node in self.function.transition_graph:
+            if isinstance(node, Function):
+                callees.add(node.addr)
+        callees.add(self.function.addr)
+
+        self.project.analyses.CompleteCallingConventions(
+            recover_variables=False,
+            prioritize_func_addrs=callees,
+            skip_other_funcs=True,
+            skip_signature_matched_functions=False,
+        )
 
     @timethis
     def _track_stack_pointers(self):
@@ -341,7 +389,7 @@ class Clinic(Analysis):
         :return:                        None
         """
 
-        blocks_by_addr_and_idx: Dict[Tuple[int,Optional[int]],ailment.Block] = { }
+        blocks_by_addr_and_idx: Dict[Tuple[int, Optional[int]], ailment.Block] = {}
 
         for ail_block in ail_graph.nodes():
             simplified = self._simplify_block(ail_block, remove_dead_memdefs=remove_dead_memdefs,
@@ -371,6 +419,7 @@ class Clinic(Analysis):
 
         simp = self.project.analyses.AILBlockSimplifier(
             ail_block,
+            self.function.addr,
             remove_dead_memdefs=remove_dead_memdefs,
             stack_pointer_tracker=stack_pointer_tracker,
             peephole_optimizations=self.peephole_optimizations,
@@ -379,7 +428,7 @@ class Clinic(Analysis):
 
     @timethis
     def _simplify_function(self, ail_graph, remove_dead_memdefs=False, stack_arg_offsets=None, unify_variables=False,
-                           max_iterations: int=8) -> None:
+                           max_iterations: int = 8) -> None:
         """
         Simplify the entire function until it reaches a fixed point.
         """
@@ -406,20 +455,26 @@ class Clinic(Analysis):
             remove_dead_memdefs=remove_dead_memdefs,
             unify_variables=unify_variables,
             stack_arg_offsets=stack_arg_offsets,
+            ail_manager=self._ail_manager,
         )
+        # cache the simplifier's RDA analysis
+        self.reaching_definitions = simp._reaching_definitions
+
         # the function graph has been updated at this point
         return simp.simplified
 
     @timethis
-    def _run_simplification_passes(self, ail_graph, stage:int=OptimizationPassStage.AFTER_GLOBAL_SIMPLIFICATION):
+    def _run_simplification_passes(self, ail_graph, stage: int = OptimizationPassStage.AFTER_GLOBAL_SIMPLIFICATION,
+                                   variable_kb=None):
 
-        addr_and_idx_to_blocks: Dict[Tuple[int,Optional[int]],ailment.Block] = { }
-        addr_to_blocks: Dict[int,Set[ailment.Block]] = defaultdict(set)
+        addr_and_idx_to_blocks: Dict[Tuple[int, Optional[int]], ailment.Block] = {}
+        addr_to_blocks: Dict[int, Set[ailment.Block]] = defaultdict(set)
 
         # update blocks_map to allow node_addr to node lookup
         def _updatedict_handler(node):
             addr_and_idx_to_blocks[(node.addr, node.idx)] = node
             addr_to_blocks[node.addr].add(node)
+
         AILGraphWalker(ail_graph, _updatedict_handler).walk()
 
         # Run each pass
@@ -428,10 +483,8 @@ class Clinic(Analysis):
             if pass_.STAGE != stage:
                 continue
 
-            analysis = getattr(self.project.analyses, pass_.__name__)
-
-            a = analysis(self.function, blocks_by_addr=addr_to_blocks, blocks_by_addr_and_idx=addr_and_idx_to_blocks,
-                         graph=ail_graph)
+            a = pass_(self.function, blocks_by_addr=addr_to_blocks, blocks_by_addr_and_idx=addr_and_idx_to_blocks,
+                         graph=ail_graph, variable_kb=variable_kb)
             if a.out_graph:
                 # use the new graph
                 ail_graph = a.out_graph
@@ -466,7 +519,7 @@ class Clinic(Analysis):
                         raise TypeError("Unsupported function argument type %s." % type(arg))
                     arg_vars.append(argvar)
             return arg_vars
-        return [ ]
+        return []
 
     @timethis
     def _make_callsites(self, ail_graph, stack_pointer_tracker=None):
@@ -495,6 +548,7 @@ class Clinic(Analysis):
                 if csm.result_block != block:
                     ail_block = csm.result_block
                     simp = self.project.analyses.AILBlockSimplifier(ail_block,
+                                                                    self.function.addr,
                                                                     stack_pointer_tracker=stack_pointer_tracker,
                                                                     peephole_optimizations=self.peephole_optimizations,
                                                                     stack_arg_offsets=csm.stack_arg_offsets,
@@ -522,6 +576,7 @@ class Clinic(Analysis):
             if block is not None \
                     and not stmt.ret_exprs \
                     and self.function.prototype is not None \
+                    and self.function.prototype.returnty is not None \
                     and type(self.function.prototype.returnty) is not SimTypeBottom:
                 new_stmt = stmt.copy()
                 ret_val = self.function.calling_convention.return_val(self.function.prototype.returnty)
@@ -538,7 +593,6 @@ class Clinic(Analysis):
                     l.warning("Unsupported type of return expression %s.", type(ret_val))
                 block.statements[stmt_idx] = new_stmt
 
-
         def _handler(block):
             walker = AILBlockWalker()
             # we don't need to handle any statement besides Returns
@@ -561,7 +615,7 @@ class Clinic(Analysis):
             return
 
         variables = variable_kb.variables[self.function.addr]
-        func_args = [ ]
+        func_args = []
         for arg in arg_list:
             func_arg = None
             arg_ty = variables.get_variable_type(arg)
@@ -601,31 +655,43 @@ class Clinic(Analysis):
         var_manager = tmp_kb.variables[self.function.addr]
         groundtruth = {}
         for variable in var_manager.variables_with_manual_types:
-            vartype = var_manager.types.get(variable, None)
+            vartype = var_manager.variable_to_types.get(variable, None)
             if vartype is not None:
-                groundtruth[vr.var_to_typevar[variable]] = vartype
+                for tv in vr.var_to_typevars[variable]:
+                    groundtruth[tv] = vartype
         # clean up existing types for this function
         var_manager.remove_types()
         # TODO: Type inference for global variables
         # run type inference
         if self._must_struct:
             must_struct = set()
-            for var, typevar in vr.var_to_typevar.items():
+            for var, typevars in vr.var_to_typevars.items():
                 if var.ident in self._must_struct:
-                    must_struct.add(typevar)
+                    must_struct |= typevars
         else:
             must_struct = None
         try:
-            tp = self.project.analyses.Typehoon(vr.type_constraints, kb=tmp_kb, var_mapping=vr.var_to_typevar,
+            tp = self.project.analyses.Typehoon(vr.type_constraints, kb=tmp_kb, var_mapping=vr.var_to_typevars,
                                                 must_struct=must_struct, ground_truth=groundtruth)
             # tp.pp_constraints()
             # tp.pp_solution()
-            tp.update_variable_types(self.function.addr, vr.var_to_typevar)
-            tp.update_variable_types('global', vr.var_to_typevar)
+            tp.update_variable_types(self.function.addr,
+                                     dict((v, t) for v, t in vr.var_to_typevars.items()
+                                          if isinstance(v, (SimRegisterVariable, SimStackVariable))))
+            tp.update_variable_types('global',
+                                     dict((v, t) for v, t in vr.var_to_typevars.items()
+                                          if isinstance(v, SimMemoryVariable) and not isinstance(v, SimStackVariable)))
         except Exception:  # pylint:disable=broad-except
             l.warning("Typehoon analysis failed. Variables will not have types. Please report to GitHub.",
                       exc_info=True)
 
+        # for any left-over variables, assign Bottom type (which will get "corrected" into a default type in
+        # VariableManager)
+        bottype = SimTypeBottom().with_arch(self.project.arch)
+        for var in var_manager._variables:
+            if var not in var_manager.variable_to_types:
+                var_manager.set_variable_type(var, bottype)
+
         # Unify SSA variables
         tmp_kb.variables.global_manager.assign_variable_names(labels=self.kb.labels, types={SimMemoryVariable})
         var_manager.unify_variables()
@@ -640,7 +706,7 @@ class Clinic(Analysis):
 
         if self._cache is not None:
             self._cache.type_constraints = vr.type_constraints
-            self._cache.var_to_typevar = vr.var_to_typevar
+            self._cache.var_to_typevar = vr.var_to_typevars
 
         return tmp_kb
 
@@ -737,19 +803,19 @@ class Clinic(Analysis):
             variables = variable_manager.find_variables_by_atom(block.addr, stmt_idx, expr)
             if len(variables) == 0:
                 # if it's a constant addr, maybe it's referencing an extern location
-                if isinstance(expr.addr, ailment.Expr.Const):
-                    symbol = self.project.loader.find_symbol(expr.addr.value)
-                    if symbol is not None:
-                        # Create a new global variable if there isn't one already
-                        global_vars = global_variables.get_global_variables(symbol.rebased_addr)
-                        if global_vars:
-                            global_var = next(iter(global_vars))
-                        else:
-                            global_var = SimMemoryVariable(symbol.rebased_addr, symbol.size, name=symbol.name)
-                            global_variables.add_variable('global', global_var.addr, global_var)
-                        expr.variable = global_var
-                        expr.variable_offset = 0
-                else:
+                base_addr, offset = self.parse_variable_addr(expr.addr)
+                if offset is not None and isinstance(offset, ailment.Expr.Expression):
+                    self._link_variables_on_expr(variable_manager, global_variables, block, stmt_idx, stmt, offset)
+                if base_addr is not None:
+                    self._link_variables_on_expr(variable_manager, global_variables, block, stmt_idx, stmt, base_addr)
+
+                # if we are accessing the variable directly (offset == 0), we link the variable onto this expression
+                if offset == 0 or (isinstance(offset, ailment.Expr.Const) and offset.value == 0):
+                    if 'reference_variable' in base_addr.tags:
+                        expr.variable = base_addr.reference_variable
+                        expr.variable_offset = base_addr.reference_variable_offset
+
+                if base_addr is None and offset is None:
                     # this is a local variable
                     self._link_variables_on_expr(variable_manager, global_variables, block, stmt_idx, stmt, expr.addr)
                     if 'reference_variable' in expr.addr.tags and expr.addr.reference_variable is not None:
@@ -813,11 +879,22 @@ class Clinic(Analysis):
 
         elif isinstance(expr, ailment.Expr.Const):
             # global variable?
-            variables = global_variables.get_global_variables(expr.value)
-            if variables:
-                var = next(iter(variables))
-                expr.tags['reference_variable'] = var
-                expr.tags['reference_variable_offset'] = None
+            global_vars = global_variables.get_global_variables(expr.value)
+            if not global_vars:
+                # detect if there is a related symbol
+                if self.project.loader.find_object_containing(expr.value):
+                    symbol = self.project.loader.find_symbol(expr.value)
+                    if symbol is not None:
+                        # Create a new global variable if there isn't one already
+                        global_vars = global_variables.get_global_variables(symbol.rebased_addr)
+                        if not global_vars:
+                            global_var = SimMemoryVariable(symbol.rebased_addr, symbol.size, name=symbol.name)
+                            global_variables.add_variable('global', global_var.addr, global_var)
+                            global_vars = {global_var}
+            if global_vars:
+                global_var = next(iter(global_vars))
+                expr.tags['reference_variable'] = global_var
+                expr.tags['reference_variable_offset'] = 0
 
         elif isinstance(expr, ailment.Stmt.Call):
             self._link_variables_on_call(variable_manager, global_variables, block, stmt_idx, expr, is_expr=True)
@@ -846,6 +923,32 @@ class Clinic(Analysis):
 
         return graph
 
+    @staticmethod
+    def _collect_externs(ail_graph, variable_kb):
+        global_vars = variable_kb.variables.global_manager.get_variables()
+        walker = AILBlockWalker()
+        variables = set()
+
+        def handle_expr(expr_idx: int, expr: ailment.expression.Load, stmt_idx: int, stmt: ailment.statement.Statement,
+                        block: Optional[ailment.Block]):
+            if expr is None:
+                return None
+            for v in [getattr(expr, 'variable', None),
+                      expr.tags.get('reference_variable', None)]:
+                if v and v in global_vars:
+                    variables.add(v)
+            return AILBlockWalker._handle_expr(walker, expr_idx, expr, stmt_idx, stmt, block)
+
+        def handle_Store(stmt_idx: int, stmt: ailment.statement.Store, block: Optional[ailment.Block]):
+            if stmt.variable and stmt.variable in global_vars:
+                variables.add(stmt.variable)
+            return AILBlockWalker._handle_Store(walker, stmt_idx, stmt, block)
+
+        walker.stmt_handlers[ailment.statement.Store] = handle_Store
+        walker._handle_expr = handle_expr
+        AILGraphWalker(ail_graph, walker.walk).walk()
+        return variables
+
     def _next_atom(self) -> int:
         return self._ail_manager.next_atom()
 
@@ -857,5 +960,20 @@ class Clinic(Analysis):
         op_type = kwargs.pop('op_type')
         return isinstance(stmt, ailment.Stmt.Call) and op_type == OP_BEFORE
 
+    def parse_variable_addr(self, addr: ailment.Expr.Expression) -> Optional[Tuple[Any,Any]]:
+        if isinstance(addr, ailment.Expr.Const):
+            return addr, 0
+        if isinstance(addr, ailment.Expr.BinaryOp):
+            if addr.op == "Add":
+                op0, op1 = addr.operands
+                if isinstance(op0, ailment.Expr.Const) \
+                        and self.project.loader.find_object_containing(op0.value) is not None:
+                    return op0, op1
+                elif isinstance(op1, ailment.Expr.Const) \
+                        and self.project.loader.find_object_containing(op1.value) is not None:
+                    return op1, op0
+                return op0, op1  # best-effort guess
+        return None, None
+
 
 register_analysis(Clinic, 'Clinic')
