@@ -1,5 +1,5 @@
 from collections import defaultdict
-from typing import Generator, Dict, Any
+from typing import Generator, Dict, Any, Optional, Set
 import operator
 import logging
 
@@ -14,7 +14,6 @@ from ...block import Block, BlockNode
 from ..cfg.cfg_utils import CFGUtils
 from .structurer_nodes import (EmptyBlockNotice, SequenceNode, CodeNode, SwitchCaseNode, BreakNode,
                                ConditionalBreakNode, LoopNode, ConditionNode, ContinueNode, CascadingConditionNode)
-from .utils import extract_jump_targets, switch_extract_cmp_bounds
 
 l = logging.getLogger(__name__)
 
@@ -28,19 +27,23 @@ class ConditionProcessor:
     """
     Convert between claripy AST and AIL expressions. Also calculates reaching conditions of all nodes on a graph.
     """
-    def __init__(self, condition_mapping=None):
+    def __init__(self, arch, condition_mapping=None):
+        self.arch = arch
         self._condition_mapping: Dict[str,Any] = {} if condition_mapping is None else condition_mapping
+        self.jump_table_conds: Dict[int,Set] = defaultdict(set)
         self.reaching_conditions = {}
         self.guarding_conditions = {}
         self._ast2annotations = {}
 
     def clear(self):
         self._condition_mapping = {}
+        self.jump_table_conds = defaultdict(set)
         self.reaching_conditions = {}
         self.guarding_conditions = {}
         self._ast2annotations = {}
 
-    def recover_reaching_conditions(self, region, with_successors=False, jump_tables=None):
+    def recover_reaching_conditions(self, region, with_successors=False,
+                                    case_entry_to_switch_head: Optional[Dict[int,int]]=None):
 
         def _strictly_postdominates(inv_idoms, node_a, node_b):
             """
@@ -66,13 +69,15 @@ class ConditionProcessor:
                     edge_conditions[edge] = predicate
                     predicate_mapping[predicate] = dst
 
-        if jump_tables:
-            self.recover_reaching_conditions_for_jumptables(region, jump_tables, edge_conditions)
-
         if with_successors and region.graph_with_successors is not None:
             _g = region.graph_with_successors
         else:
             _g = region.graph
+
+        # special handling for jump table entries - do not allow crossing between cases
+        if case_entry_to_switch_head:
+            _g = self._remove_crossing_edges_between_cases(_g, case_entry_to_switch_head)
+
         end_nodes = {n for n in _g.nodes() if _g.out_degree(n) == 0}
         inverted_graph: networkx.DiGraph = shallow_reverse(_g)
         if end_nodes:
@@ -94,6 +99,15 @@ class ConditionProcessor:
         sorted_nodes = CFGUtils.quasi_topological_sort_nodes(_g)
         terminating_nodes = [ ]
         for node in sorted_nodes:
+            # create special conditions for all nodes that are jump table entries
+            if case_entry_to_switch_head:
+                if node.addr in case_entry_to_switch_head:
+                    jump_target_var = self.create_jump_target_var(case_entry_to_switch_head[node.addr])
+                    cond = jump_target_var == claripy.BVV(node.addr, self.arch.bits)
+                    reaching_conditions[node] = cond
+                    self.jump_table_conds[case_entry_to_switch_head[node.addr]].add(cond)
+                    continue
+
             preds = _g.predecessors(node)
             reaching_condition = None
 
@@ -121,23 +135,7 @@ class ConditionProcessor:
             if reaching_condition is not None:
                 reaching_conditions[node] = self.simplify_condition(reaching_condition)
 
-        # My hypothesis to be proved: in any regioned graph, there must be a node with a 0 out-degree whose reaching
-        # conditions can be marked as True. In other words, if all 0 out-degree nodes have non-trivial reaching
-        # conditions, we can always pick one of them and change its reaching condition to True, without changing the
-        # semantics of the regioned graph.
-        if terminating_nodes and all(not reaching_conditions[node].is_true() for node in terminating_nodes
-                                     if node in reaching_conditions):
-            # pick the node with the greatest in-degree
-            terminating_nodes = sorted(terminating_nodes, key=_g.in_degree)
-            node_with_greatest_indegree = terminating_nodes[-1]
-            if _g.in_degree(node_with_greatest_indegree) > 1:
-                # forcing the in-degree to be greater than 1 allows us to skip the case blocks in switch-cases
-                # otherwise structurer will fail to structure the control flow
-                reaching_conditions[node_with_greatest_indegree] = claripy.true
-                l.warning("Marking node %r as trivially reachable. Disable this optimization in condition_processor.py "
-                          "if it leads to incorrect decompilation result.", node_with_greatest_indegree)
-
-        # Another hypothesis: for nodes where two paths come together *and* those that cannot be further structured into
+        # My hypothesis: for nodes where two paths come together *and* those that cannot be further structured into
         # another if-else construct (we take the short-cut by testing if the operator is an "Or" after running our
         # condition simplifiers previously), we are better off using their "guarding conditions" instead of their
         # reaching conditions for if-else. see my super long chatlog with rhelmot on 5/14/2021.
@@ -180,43 +178,6 @@ class ConditionProcessor:
         self.reaching_conditions = reaching_conditions
         self.guarding_conditions = guarding_conditions
 
-    def recover_reaching_conditions_for_jumptables(self, region, jump_tables, edge_conditions):
-
-        addr2nodes = dict((node.addr, node) for node in region.graph.nodes())
-
-        # special handling for jump tables
-        for src in region.graph.nodes():
-            try:
-                last_stmt = self.get_last_statement(src)
-            except EmptyBlockNotice:
-                continue
-            successor_addrs = extract_jump_targets(last_stmt)
-            if len(successor_addrs) != 2:
-                continue
-
-            for t in successor_addrs:
-                if t in addr2nodes and t in jump_tables:
-                    # this is a candidate!
-                    target = t
-                    break
-            else:
-                continue
-
-            cmp = switch_extract_cmp_bounds(last_stmt)
-            if not cmp:
-                continue
-
-            cmp_expr, cmp_lb, cmp_ub = cmp  # pylint:disable=unused-variable
-
-            jump_table = jump_tables[target]
-            node_a = addr2nodes[target]
-
-            # edge conditions
-            for i, entry_addr in enumerate(jump_table.jumptable_entries):
-                cond = self.claripy_ast_from_ail_condition(cmp_expr) == i + cmp_lb
-                edge = node_a, addr2nodes[entry_addr]
-                edge_conditions[edge] = cond
-
     def remove_claripy_bool_asts(self, node, memo=None):
 
         # Convert claripy Bool ASTs to AIL expressions
@@ -521,9 +482,11 @@ class ConditionProcessor:
         if isinstance(cond, ailment.Expr.Expression):
             return cond
 
-        if cond.op == "BoolS" and claripy.is_true(cond):
-            return cond
-        if cond.args[0] in self._condition_mapping:
+        if cond.op in {"BoolS", "BoolV"} and claripy.is_true(cond):
+            return ailment.Expr.Const(None, None, True, 1)
+        if cond in self._condition_mapping:
+            return self._condition_mapping[cond]
+        if cond.op in {"BVS", "BoolS"} and cond.args[0] in self._condition_mapping:
             return self._condition_mapping[cond.args[0]]
 
         def _binary_op_reduce(op, args, signed=False):
@@ -567,6 +530,7 @@ class ConditionProcessor:
             '__and__': lambda cond_: _binary_op_reduce('And', cond_.args),
             '__lshift__': lambda cond_: _binary_op_reduce('Shl', cond_.args),
             '__rshift__': lambda cond_: _binary_op_reduce('Sar', cond_.args),
+            '__floordiv__': lambda cond_: _binary_op_reduce('Div', cond_.args),
             'LShR': lambda cond_: _binary_op_reduce('Shr', cond_.args),
             'BVV': lambda cond_: ailment.Expr.Const(None, None, cond_.args[0], cond_.size()),
             'BoolV': lambda cond_: ailment.Expr.Const(None, None, True, 1) if cond_.args[0] is True
@@ -634,11 +598,17 @@ class ConditionProcessor:
             'Mulls': lambda expr, _: _dummy_bvs(expr),
         }
 
-        if isinstance(condition, (ailment.Expr.Load, ailment.Expr.DirtyExpression, ailment.Expr.BasePointerOffset,
+        if isinstance(condition, (ailment.Expr.DirtyExpression, ailment.Expr.BasePointerOffset,
                                   ailment.Expr.ITE, ailment.Stmt.Call)):
             return _dummy_bvs(condition)
-        elif isinstance(condition, ailment.Expr.Register):
-            var = claripy.BVS('ailexpr_%s-%d' % (repr(condition), condition.idx), condition.bits, explicit_name=True)
+        elif isinstance(condition, (ailment.Expr.Load, ailment.Expr.Register)):
+            # does it have a variable associated?
+            if condition.variable is not None:
+                var = claripy.BVS('ailexpr_%s-%s' % (repr(condition), condition.variable.name), condition.bits,
+                                  explicit_name=True)
+            else:
+                var = claripy.BVS('ailexpr_%s-%d' % (repr(condition), condition.idx), condition.bits,
+                                  explicit_name=True)
             self._condition_mapping[var.args[0]] = condition
             return var
         elif isinstance(condition, ailment.Expr.Convert):
@@ -667,8 +637,11 @@ class ConditionProcessor:
 
         lambda_expr = _mapping.get(condition.verbose_op, None)
         if lambda_expr is None:
-            raise NotImplementedError("Unsupported AIL expression operation %s. Consider implementing."
-                                      % condition.verbose_op)
+            # fall back to op
+            lambda_expr = _mapping.get(condition.op, None)
+        if lambda_expr is None:
+            raise NotImplementedError("Unsupported AIL expression operation %s or %s. Consider implementing."
+                                      % (condition.op, condition.verbose_op))
         r = lambda_expr(condition, self.claripy_ast_from_ail_condition)
         if r is NotImplemented:
             r = claripy.BVS("ailexpr_%r" % condition, condition.bits, explicit_name=True)
@@ -719,10 +692,13 @@ class ConditionProcessor:
         raise RuntimeError("Unreachable reached")
 
     @staticmethod
-    def simplify_condition(cond):
+    def simplify_condition(cond, depth_limit=8, variables_limit=8):
         memo = { }
+        if cond.depth > depth_limit or len(cond.variables) > variables_limit:
+            return cond
         sympy_expr = ConditionProcessor.claripy_ast_to_sympy_expr(cond, memo=memo)
-        return ConditionProcessor.sympy_expr_to_claripy_ast(sympy.simplify_logic(sympy_expr), memo)
+        r = ConditionProcessor.sympy_expr_to_claripy_ast(sympy.simplify_logic(sympy_expr, deep=False), memo)
+        return r
 
     @staticmethod
     def simplify_condition_deprecated(cond):
@@ -991,6 +967,56 @@ class ConditionProcessor:
             # TODO: Finish the implementation
             print(term, "is redundant")
 
+    #
+    # Graph processing
+    #
+
+    @staticmethod
+    def _remove_crossing_edges_between_cases(graph: networkx.DiGraph,
+                                             case_entry_to_switch_head: Dict[int,int]) -> networkx.DiGraph:
+        starting_nodes = { node for node in graph if node.addr in case_entry_to_switch_head }
+        if not starting_nodes:
+            return graph
+
+        traversed_nodes = set()
+        edges_to_remove = set()
+        for starting_node in starting_nodes:
+
+            queue = [starting_node]
+            while queue:
+                src = queue.pop(0)
+                traversed_nodes.add(src)
+                successors = graph.successors(src)
+                for succ in successors:
+                    if succ in traversed_nodes:
+                        # we should not traverse this node twice
+                        if graph.out_degree(succ) > 0:
+                            edges_to_remove.add((src, succ))
+                        continue
+                    if succ in starting_nodes:
+                        # we do not want any jump from one node to a starting node
+                        edges_to_remove.add((src, succ))
+                        continue
+                    traversed_nodes.add(src)
+                    queue.append(succ)
+
+        if not edges_to_remove:
+            return graph
+
+        # make a copy before modifying the graph
+        graph = networkx.DiGraph(graph)
+        graph.remove_edges_from(edges_to_remove)
+        return graph
+
+    #
+    # Utils
+    #
+
+    def create_jump_target_var(self, jumptable_head_addr: int):
+        return claripy.BVS("jump_table_%x" % jumptable_head_addr,
+                           self.arch.bits,
+                           explicit_name=True)
+
 
 # delayed import
 from .region_identifier import GraphRegion, MultiNode  # pylint:disable=wrong-import-position
