@@ -50,6 +50,8 @@ class RegionSimplifier(Analysis):
         r = self._simplify_cascading_ifs(r)
         #
         r = self._simplify_loops(r)
+        # Remove empty nodes again
+        r = self._remove_empty_nodes(r)
 
         self.result = r
 
@@ -58,10 +60,12 @@ class RegionSimplifier(Analysis):
     #
 
     def _fold_oneuse_expressions(self, region):
-        expr_counter = ExpressionCounter(region, self.variable_kb.variables[self.func.addr])
+        variable_manager = self.variable_kb.variables[self.func.addr]
+        expr_counter = ExpressionCounter(region, variable_manager)
 
         variable_assignments = {}
         variable_uses = {}
+        variable_assignment_dependencies = {}
         for var, uses in expr_counter.uses.items():
             if len(uses) == 1 \
                     and var in expr_counter.assignments \
@@ -71,6 +75,8 @@ class RegionSimplifier(Analysis):
                 # make sure all variables that var depends on has only been assigned once
                 fail = False
                 for dep_var in deps:
+                    if dep_var.is_function_argument:
+                        continue
                     if dep_var not in expr_counter.assignments or len(expr_counter.assignments[dep_var]) != 1:
                         fail = True
                         break
@@ -80,18 +86,27 @@ class RegionSimplifier(Analysis):
                 if isinstance(definition, ailment.Stmt.Call):
                     # clear the existing variable since we no longer write to this variable after expression folding
                     definition = definition.copy()
-                    definition.ret_expr = definition.ret_expr.copy()
-                    definition.ret_expr.variable = None
+                    if definition.ret_expr is not None:
+                        definition.ret_expr = definition.ret_expr.copy()
+                        definition.ret_expr.variable = None
                 variable_assignments[var] = definition, loc
                 variable_uses[var] = next(iter(expr_counter.uses[var]))
+                variable_assignment_dependencies[var] = deps
+
+        # any variable definition that uses an existing to-be-removed variable cannot be folded
+        all_variables_to_fold = set(variable_assignments)
+        for var in all_variables_to_fold:
+            if all_variables_to_fold.intersection(variable_assignment_dependencies[var]):
+                del variable_assignments[var]
+                del variable_uses[var]
 
         # replace them
-        ExpressionFolder(variable_assignments, variable_uses, region)
+        ExpressionFolder(variable_assignments, variable_uses, region, variable_manager)
         return region
 
     @staticmethod
     def _remove_empty_nodes(region):
-        return EmptyNodeRemover(region).result
+        return EmptyNodeRemover(region, claripy_ast_conditions=False).result
 
     @staticmethod
     def _transform_to_cascading_ifs(region):
