@@ -1,6 +1,6 @@
 # pylint:disable=isinstance-second-argument-not-valid-type
 import weakref
-from typing import Set, Optional, Any, Tuple, Union, TYPE_CHECKING
+from typing import Set, Optional, Any, Tuple, Union, List
 from collections import defaultdict
 import logging
 
@@ -12,6 +12,7 @@ from ... import sim_options
 from ...storage.memory_mixins import LabeledMemory
 from ...errors import SimMemoryMissingError
 from ...code_location import CodeLocation  # pylint:disable=unused-import
+from ...storage.memory_object import SimMemoryObject, SimLabeledMemoryObject
 from .. import register_analysis
 from ..analysis import Analysis
 from ..forward_analysis import ForwardAnalysis, FunctionGraphVisitor, SingleNodeGraphVisitor
@@ -19,9 +20,6 @@ from .engine_vex import SimEnginePropagatorVEX
 from .engine_ail import SimEnginePropagatorAIL
 from .prop_value import PropValue
 
-if TYPE_CHECKING:
-    from angr.storage import SimMemoryObject
-
 
 _l = logging.getLogger(name=__name__)
 
@@ -57,12 +55,19 @@ class PropagatorState:
         return weakref.proxy(self)
 
     @staticmethod
-    def _mo_cmp(mo_self: 'SimMemoryObject', mo_other: 'SimMemoryObject', addr: int, size: int):  # pylint:disable=unused-argument
+    def _mo_cmp(mo_self: Union['SimMemoryObject','SimLabeledMemoryObject'],
+                mo_other: Union['SimMemoryObject','SimLabeledMemoryObject'],
+                addr: int, size: int):  # pylint:disable=unused-argument
         # comparing bytes from two sets of memory objects
         # we don't need to resort to byte-level comparison. object-level is good enough.
 
         if mo_self.object.symbolic or mo_other.object.symbolic:
-            return mo_self.object is mo_other.object
+            if type(mo_self) is SimLabeledMemoryObject and type(mo_other) is SimLabeledMemoryObject:
+                return mo_self.label == mo_other.label and mo_self.object is mo_other.object
+            if type(mo_self) is SimMemoryObject and type(mo_other) is SimMemoryObject:
+                return mo_self.object is mo_other.object
+            # SimMemoryObject vs SimLabeledMemoryObject -> the label must be different
+            return False
         return None
 
     @staticmethod
@@ -263,7 +268,8 @@ class PropagatorAILState(PropagatorState):
     Describes the state used in the AIL engine of Propagator.
     """
 
-    __slots__ = ('_registers', '_stack_variables', '_tmps', '_inside_call_stmt')
+    __slots__ = ('_registers', '_stack_variables', '_tmps', 'temp_expressions', 'register_expressions',
+                 '_inside_call_stmt', 'last_stack_store', 'global_stores')
 
     def __init__(self, arch, project=None, replacements=None, only_consts=False, prop_count=None, equivalence=None,
                  stack_variables=None, registers=None):
@@ -278,10 +284,14 @@ class PropagatorAILState(PropagatorState):
         self._registers = LabeledMemory(memory_id='reg', top_func=self.top, page_kwargs={'mo_cmp': self._mo_cmp}) \
             if registers is None else registers
         self._tmps = {}
+        self.temp_expressions = { }
+        self.register_expressions = { }
         self._inside_call_stmt = False  # temporary variable that is only used internally
 
         self._registers.set_state(self)
         self._stack_variables.set_state(self)
+        self.last_stack_store: Optional[Tuple[int,int,ailment.Stmt.Store]] = None
+        self.global_stores: List[Tuple[int,int,Any,ailment.Stmt.Store]] = [ ]
 
     def __repr__(self):
         return "<PropagatorAILState>"
@@ -321,7 +331,8 @@ class PropagatorAILState(PropagatorState):
             return
 
         for offset, chopped_value, size, label in value.value_and_labels():
-            self._registers.store(reg.reg_offset + offset, chopped_value, size=size, label=label)
+            self._registers.store(reg.reg_offset + offset, chopped_value, size=size, label=label,
+                                  endness=self.project.arch.register_endness)
 
     def store_stack_variable(self, sp_offset: int, new: PropValue, endness=None) -> None:  # pylint:disable=unused-argument
         # normalize sp_offset to handle negative offsets
@@ -333,7 +344,8 @@ class PropagatorAILState(PropagatorState):
 
     def load_register(self, reg: ailment.Expr.Register) -> Optional[PropValue]:
         try:
-            value, labels = self._registers.load_with_labels(reg.reg_offset, size=reg.size)
+            value, labels = self._registers.load_with_labels(reg.reg_offset, size=reg.size,
+                                                             endness=self.project.arch.register_endness)
         except SimMemoryMissingError:
             # value does not exist
             return None
@@ -512,15 +524,31 @@ class PropagatorAnalysis(ForwardAnalysis, Analysis):  # pylint:disable=abstract-
         if isinstance(node, ailment.Block):
             # AIL
             state = PropagatorAILState(self.project.arch, project=self.project, only_consts=self._only_consts)
+            ail = True
         else:
             # VEX
             state = PropagatorVEXState(self.project.arch, project=self.project, only_consts=self._only_consts,
                                        do_binops=self._do_binops, store_tops=self._store_tops)
             spoffset_var = self._engine_vex.sp_offset(0)
+            ail = False
             state.store_register(self.project.arch.sp_offset,
                                  self.project.arch.bytes,
                                  spoffset_var,
                                  )
+
+        if self.project.arch.name == "MIPS64":
+            if self._function is not None:
+                if ail:
+                    state.store_register(ailment.Expr.Register(None, None, self.project.arch.registers['t9'][0],
+                                                               self.project.arch.registers['t9'][0]),
+                                         PropValue(claripy.BVV(self._function.addr, 64)),
+                                         )
+                else:
+                    state.store_register(self.project.arch.registers['t9'][0],  # pylint:disable=too-many-function-args
+                                         self.project.arch.registers['t9'][1],
+                                         claripy.BVV(self._function.addr, 64),
+                                         )
+
         self._initial_state = state
         return state
 
