@@ -146,8 +146,10 @@ class SimEngineRDAIL(
     def _ail_handle_ConditionalJump(self, stmt):
 
         _ = self._expr(stmt.condition)  # pylint:disable=unused-variable
-        _ = self._expr(stmt.true_target)  # pylint:disable=unused-variable
-        _ = self._expr(stmt.false_target)  # pylint:disable=unused-variable
+        if stmt.true_target is not None:
+            _ = self._expr(stmt.true_target)  # pylint:disable=unused-variable
+        if stmt.false_target is not None:
+            _ = self._expr(stmt.false_target)  # pylint:disable=unused-variable
 
         ip = Register(self.arch.ip_offset, self.arch.bytes)
         self.state.kill_definitions(ip)
@@ -156,7 +158,10 @@ class SimEngineRDAIL(
         self._handle_Call_base(stmt, is_expr=False)
 
     def _handle_Call_base(self, stmt: ailment.Stmt.Call, is_expr: bool=False):
-        target = self._expr(stmt.target)  # pylint:disable=unused-variable
+        if isinstance(stmt.target, ailment.Expr.Expression):
+            target = self._expr(stmt.target)  # pylint:disable=unused-variable
+        else:
+            target = stmt.target
         codeloc = self._codeloc()
 
         ip = Register(self.arch.ip_offset, self.arch.bytes)
@@ -200,11 +205,13 @@ class SimEngineRDAIL(
 
         # Add definition
         return_reg_offset = None
+        # TODO: Expose it as an option
+        return_value_use_full_width_reg = True
         if not is_expr:
             if stmt.ret_expr is not None:
                 if isinstance(stmt.ret_expr, ailment.Expr.Register):
                     return_reg_offset = stmt.ret_expr.reg_offset
-                    return_reg_size = stmt.ret_expr.size
+                    return_reg_size = stmt.ret_expr.size if not return_value_use_full_width_reg else self.arch.bytes
                     reg_atom = Register(return_reg_offset, return_reg_size)
                     top = self.state.top(return_reg_size * self.arch.byte_width)
                     self.state.kill_and_add_definition(reg_atom, codeloc, MultiValues(offset_to_values={0: {top}}))
@@ -327,7 +334,7 @@ class SimEngineRDAIL(
         self._handle_Call_base(expr, is_expr=True)
         return MultiValues(offset_to_values={0: {self.state.top(expr.bits)}})
 
-    def _ail_handle_Register(self, expr) -> MultiValues:
+    def _ail_handle_Register(self, expr: ailment.Expr.Register) -> MultiValues:
 
         self.state: ReachingDefinitionsState
 
@@ -364,7 +371,7 @@ class SimEngineRDAIL(
         else:
             codeloc = self._codeloc()
             for def_ in defs:
-                self.state.add_use_by_def(def_, codeloc)
+                self.state.add_use_by_def(def_, codeloc, expr=expr)
 
         return value
 
@@ -389,7 +396,7 @@ class SimEngineRDAIL(
             dummy_atom = MemoryLocation(0, size, endness=expr.endness)
             top = self.state.annotate_with_def(top, Definition(dummy_atom, ExternalCodeLocation()))
             # add use
-            self.state.add_use(dummy_atom, self._codeloc())
+            self.state.add_use(dummy_atom, self._codeloc(), expr=expr)
             return MultiValues(offset_to_values={0: {top}})
 
         result: Optional[MultiValues] = None
@@ -398,14 +405,14 @@ class SimEngineRDAIL(
                 continue
             if addr.concrete:
                 # a concrete address
-                addr = addr._model_concrete.value
+                concrete_addr: int = addr._model_concrete.value
                 try:
-                    vs: MultiValues = self.state.memory_definitions.load(addr, size=size, endness=expr.endness)
+                    vs: MultiValues = self.state.memory_definitions.load(concrete_addr, size=size, endness=expr.endness)
                 except SimMemoryMissingError:
                     continue
 
-                memory_location = MemoryLocation(addr, size, endness=expr.endness)
-                self.state.add_use(memory_location, self._codeloc())
+                memory_location = MemoryLocation(concrete_addr, size, endness=expr.endness)
+                self.state.add_use(memory_location, self._codeloc(), expr=expr)
                 result = result.merge(vs) if result is not None else vs
             elif self.state.is_stack_address(addr):
                 stack_offset = self.state.get_stack_offset(addr)
@@ -417,7 +424,7 @@ class SimEngineRDAIL(
                         continue
 
                     memory_location = MemoryLocation(SpOffset(self.arch.bits, stack_offset), size, endness=expr.endness)
-                    self.state.add_use(memory_location, self._codeloc())
+                    self.state.add_use(memory_location, self._codeloc(), expr=expr)
                     result = result.merge(vs) if result is not None else vs
             else:
                 l.debug('Memory address %r undefined or unsupported at pc %#x.', addr, self.ins_addr)
@@ -442,7 +449,7 @@ class SimEngineRDAIL(
             dummy_atom = MemoryLocation(0, size, endness=self.arch.memory_endness)
             top = self.state.annotate_with_def(top, Definition(dummy_atom, ExternalCodeLocation()))
             # add use
-            self.state.add_use(dummy_atom, self._codeloc())
+            self.state.add_use(dummy_atom, self._codeloc(), expr=expr)
             return MultiValues(offset_to_values={0: {top}})
 
         converted = set()
@@ -477,7 +484,9 @@ class SimEngineRDAIL(
         operand: MultiValues = self._expr(expr.operand)
         bits = expr.bits
 
-        r = None
+        if operand is None:
+            return MultiValues(offset_to_values={0: {self.state.top(bits)}})
+
         operand_v = operand.one_value()
 
         if operand_v is not None and operand_v.concrete:
@@ -487,6 +496,22 @@ class SimEngineRDAIL(
 
         return r
 
+    def _ail_handle_Neg(self, expr: ailment.Expr.UnaryOp) -> MultiValues:
+        operand: MultiValues = self._expr(expr.operand)
+        bits = expr.bits
+
+        if operand is None:
+            return MultiValues(offset_to_values={0: {self.state.top(bits)}})
+
+        operand_v = operand.one_value()
+
+        if operand_v is not None and operand_v.concrete:
+            r = MultiValues(offset_to_values={0: {-operand_v}})
+        else:
+            r = MultiValues(offset_to_values={0: {self.state.top(bits)}})
+
+        return r
+
     def _ail_handle_BinaryOp(self, expr: ailment.Expr.BinaryOp) -> MultiValues:
         r = super()._ail_handle_BinaryOp(expr)
         if isinstance(r, ailment.Expr.BinaryOp):
@@ -560,6 +585,53 @@ class SimEngineRDAIL(
 
         return r
 
+    def _ail_handle_Div(self, expr):
+
+        arg0, arg1 = expr.operands
+
+        self._expr(arg0)
+        self._expr(arg1)
+        bits = expr.bits
+
+        r = MultiValues(offset_to_values={0: {self.state.top(bits)}})
+        return r
+
+    def _ail_handle_DivMod(self, expr):
+        return self._ail_handle_Div(expr)
+
+    def _ail_handle_Mul(self, expr):
+
+        arg0, arg1 = expr.operands
+
+        self._expr(arg0)
+        self._expr(arg1)
+        bits = expr.bits
+
+        r = MultiValues(offset_to_values={0: {self.state.top(bits)}})
+        return r
+
+    def _ail_handle_Mull(self, expr):
+
+        arg0, arg1 = expr.operands
+
+        self._expr(arg0)
+        self._expr(arg1)
+        bits = expr.bits
+
+        r = MultiValues(offset_to_values={0: {self.state.top(bits)}})
+        return r
+
+    def _ail_handle_Mod(self, expr):
+
+        arg0, arg1 = expr.operands
+
+        self._expr(arg0)
+        self._expr(arg1)
+        bits = expr.bits
+
+        r = MultiValues(offset_to_values={0: {self.state.top(bits)}})
+        return r
+
     def _ail_handle_Shr(self, expr: ailment.Expr.BinaryOp) -> MultiValues:
         expr0: MultiValues = self._expr(expr.operands[0])
         expr1: MultiValues = self._expr(expr.operands[1])
@@ -732,6 +804,40 @@ class SimEngineRDAIL(
 
         return r
 
+    def _ail_handle_LogicalAnd(self, expr: ailment.Expr.BinaryOp) -> MultiValues:
+        expr0: MultiValues = self._expr(expr.operands[0])
+        expr1: MultiValues = self._expr(expr.operands[1])
+        bits = expr.bits
+
+        r = None
+        expr0_v = expr0.one_value()
+        expr1_v = expr1.one_value()
+
+        # TODO: can maybe be smarter about this. if we can determine that expr0 is never falsey, we can just return it,
+        # TODO: or if it's always falsey we can return expr1 (did I get this backwards?)
+        if expr0_v is None or expr1_v is None:
+            r = MultiValues(offset_to_values={0: {self.state.top(bits)}})
+            return r
+
+        r = MultiValues(offset_to_values={0: {claripy.If(expr0_v == 0, expr0_v, expr1_v)}})
+        return r
+
+    def _ail_handle_LogicalOr(self, expr: ailment.Expr.BinaryOp) -> MultiValues:
+        expr0: MultiValues = self._expr(expr.operands[0])
+        expr1: MultiValues = self._expr(expr.operands[1])
+        bits = expr.bits
+
+        r = None
+        expr0_v = expr0.one_value()
+        expr1_v = expr1.one_value()
+
+        if expr0_v is None or expr1_v is None:
+            r = MultiValues(offset_to_values={0: {self.state.top(bits)}})
+            return r
+
+        r = MultiValues(offset_to_values={0: {claripy.If(expr0_v != 0, expr0_v, expr1_v)}})
+        return r
+
     def _ail_handle_Xor(self, expr: ailment.Expr.BinaryOp) -> MultiValues:
         expr0: MultiValues = self._expr(expr.operands[0])
         expr1: MultiValues = self._expr(expr.operands[1])
