@@ -459,7 +459,10 @@ class SimEngineRDVEX(
                 if bits > v.size():
                     data.add(v.zero_extend(bits - v.size()))
                 else:
-                    data.add(v[bits - 1:0])
+                    if isinstance(v, claripy.ast.fp.FP):
+                        data.add(v.val_to_bv(bits))
+                    else:
+                        data.add(v[bits - 1:0])
             r = MultiValues(offset_to_values={next(iter(arg_0.values.keys())): data})
 
         else:
@@ -537,12 +540,13 @@ class SimEngineRDVEX(
         elif expr0_v is None and expr1_v is not None:
             # adding a single value to a multivalue
             if len(expr0.values) == 1 and 0 in expr0.values:
-                vs = {v + expr1_v for v in expr0.values[0]}
+
+                vs = {v.sign_extend(expr1_v.size() - v.size()) + expr1_v for v in expr0.values[0]}
                 r = MultiValues(offset_to_values={0: vs})
         elif expr0_v is not None and expr1_v is None:
             # adding a single value to a multivalue
             if len(expr1.values) == 1 and 0 in expr1.values:
-                vs = {v + expr0_v for v in expr1.values[0]}
+                vs = {v.sign_extend(expr0_v.size() - v.size()) + expr0_v for v in expr1.values[0]}
                 r = MultiValues(offset_to_values={0: vs})
         else:
             # adding two single values together
@@ -640,7 +644,10 @@ class SimEngineRDVEX(
         else:
             if expr0_v.concrete and expr1_v.concrete:
                 # dividing two single values
-                r = MultiValues(offset_to_values={0: {expr0_v / expr1_v}})
+                if expr1_v._model_concrete.value == 0:
+                    r = MultiValues(offset_to_values={0: {self.state.top(bits)}})
+                else:
+                    r = MultiValues(offset_to_values={0: {expr0_v / expr1_v}})
 
         if r is None:
             r = MultiValues(offset_to_values={0: {self.state.top(bits)}})
@@ -700,12 +707,12 @@ class SimEngineRDVEX(
         elif expr0_v is None and expr1_v is not None:
             # bitwise-xor a single value with a multivalue
             if len(expr0.values) == 1 and 0 in expr0.values:
-                vs = {v ^ expr1_v for v in expr0.values[0]}
+                vs = {v.sign_extend(expr1_v.size() - v.size()) ^ expr1_v for v in expr0.values[0]}
                 r = MultiValues(offset_to_values={0: vs})
         elif expr0_v is not None and expr1_v is None:
             # bitwise-xor a single value to a multivalue
             if len(expr1.values) == 1 and 0 in expr1.values:
-                vs = {v ^ expr0_v for v in expr1.values[0]}
+                vs = {v.sign_extend(expr0_v.size() - v.size()) ^ expr0_v for v in expr1.values[0]}
                 r = MultiValues(offset_to_values={0: vs})
         else:
             if expr0_v.concrete and expr1_v.concrete:
@@ -1017,7 +1024,9 @@ class SimEngineRDVEX(
         executed_rda = False
         if symbol is not None:
             codeloc = CodeLocation(func_addr_int, 0, None, func_addr_int, context=self._context)
-            executed_rda, state = self._function_handler.handle_external_function_symbol(self.state, symbol=symbol, src_codeloc=codeloc)
+            executed_rda, state = self._function_handler.handle_external_function_symbol(self.state,
+                                                                                         symbol=symbol,
+                                                                                         src_codeloc=codeloc)
             self.state = state
 
         elif is_internal is True:
@@ -1082,13 +1091,16 @@ class SimEngineRDVEX(
                     self._tag_definitions_of_atom(atom, func_addr_int)
                 elif isinstance(arg, SimStructArg):
                     min_stack_offset = None
-                    for subargfield, subargloc in arg.locs.items():
-                        if not isinstance(subargloc, SimStackArg):
-                            raise TypeError(f"Unexpected: Field {subargfield} in {arg} is not a stack location.")
-                        if min_stack_offset is None:
-                            min_stack_offset = subargloc.stack_offset
-                        elif min_stack_offset > subargloc.stack_offset:
-                            min_stack_offset = subargloc.stack_offset
+                    for _, subargloc in arg.locs.items():
+                        if isinstance(subargloc, SimStackArg):
+                            if min_stack_offset is None:
+                                min_stack_offset = subargloc.stack_offset
+                            elif min_stack_offset > subargloc.stack_offset:
+                                min_stack_offset = subargloc.stack_offset
+                        elif isinstance(subargloc, SimRegArg):
+                            atom = Register(subargloc.reg_offset, subargloc.size)
+                            self.state.add_use(atom, code_loc)
+                            self._tag_definitions_of_atom(atom, func_addr_int)
 
                     if min_stack_offset is not None:
                         atom = MemoryLocation(SpOffset(self.arch.bits,
