@@ -1,10 +1,11 @@
-from typing import Optional, Iterable, Set, Generator, Tuple, TYPE_CHECKING
+from typing import Optional, Iterable, Set, Generator, Tuple, Any, TYPE_CHECKING
 import logging
 
 import archinfo
 import claripy
 
 from ...storage.memory_mixins.paged_memory.pages.multi_values import MultiValues
+from ...storage.memory_mixins import MultiValuedMemory
 from ...knowledge_plugins.key_definitions import LiveDefinitions
 from ...knowledge_plugins.key_definitions.atoms import Atom, GuardUse, Register, MemoryLocation, FunctionCall, \
     ConstantSrc
@@ -132,7 +133,7 @@ class ReachingDefinitionsState:
         else:
             raise ValueError("Unsupported architecture word size %d" % self.arch.bits)
 
-    def annotate_with_def(self, symvar: claripy.ast.Base, definition: Definition):
+    def annotate_with_def(self, symvar: claripy.ast.Base, definition: Definition) -> claripy.ast.Base:
         """
 
         :param symvar:
@@ -161,11 +162,11 @@ class ReachingDefinitionsState:
         return self.live_definitions.register_uses
 
     @property
-    def register_definitions(self):
+    def register_definitions(self) -> MultiValuedMemory:
         return self.live_definitions.register_definitions
 
     @property
-    def stack_definitions(self):
+    def stack_definitions(self) -> MultiValuedMemory:
         return self.live_definitions.stack_definitions
 
     @property
@@ -173,7 +174,7 @@ class ReachingDefinitionsState:
         return self.live_definitions.stack_uses
 
     @property
-    def heap_definitions(self):
+    def heap_definitions(self) -> MultiValuedMemory:
         return self.live_definitions.heap_definitions
 
     @property
@@ -185,7 +186,7 @@ class ReachingDefinitionsState:
         return self.live_definitions.memory_uses
 
     @property
-    def memory_definitions(self):
+    def memory_definitions(self) -> MultiValuedMemory:
         return self.live_definitions.memory_definitions
 
     @property
@@ -249,28 +250,32 @@ class ReachingDefinitionsState:
             t9_def = Definition(t9_atom, ExternalCodeLocation(), tags={InitialValueTag()})
             t9 = self.annotate_with_def(claripy.BVV(func_addr, self.arch.bits), t9_def)
             self.register_definitions.store(t9_offset, t9)
-
         if cc is not None:
-            for arg in cc.arg_locs(self.analysis.kb.functions[func_addr].prototype):
-                # initialize register parameters
-                if isinstance(arg, SimRegArg):
-                    # FIXME: implement reg_offset handling in SimRegArg
-                    reg_offset = self.arch.registers[arg.reg_name][0]
-                    reg_atom = Register(reg_offset, self.arch.bytes)
-                    reg_def = Definition(reg_atom, ExternalCodeLocation(), tags={ParameterTag()})
-                    reg = self.annotate_with_def(self.top(self.arch.bits), reg_def)
-                    self.register_definitions.store(reg_offset, reg)
+            prototype = self.analysis.kb.functions[func_addr].prototype
+            if prototype is not None:
+                for loc in cc.arg_locs(prototype):
+                    for arg in loc.get_footprint():
+                        # initialize register parameters
+                        if isinstance(arg, SimRegArg):
+                            # FIXME: implement reg_offset handling in SimRegArg
+                            reg_offset = self.arch.registers[arg.reg_name][0]
+                            reg_atom = Register(reg_offset, self.arch.bytes)
+                            reg_def = Definition(reg_atom, ExternalCodeLocation(),
+                                                 tags={ParameterTag(function=func_addr)})
+                            reg = self.annotate_with_def(self.top(self.arch.bits), reg_def)
+                            self.register_definitions.store(reg_offset, reg)
 
-                # initialize stack parameters
-                elif isinstance(arg, SimStackArg):
-                    ml_atom = MemoryLocation(SpOffset(self.arch.bits, arg.stack_offset), arg.size)
-                    ml_def = Definition(ml_atom, ExternalCodeLocation(), tags={ParameterTag()})
-                    ml = self.annotate_with_def(self.top(self.arch.bits), ml_def)
-                    self.stack_definitions.store(self._initial_stack_pointer() + arg.stack_offset, ml,
-                                                 endness=self.arch.memory_endness)
-
-                else:
-                    raise TypeError('Unsupported parameter type %s.' % type(arg).__name__)
+                        # initialize stack parameters
+                        elif isinstance(arg, SimStackArg):
+                            ml_atom = MemoryLocation(SpOffset(self.arch.bits, arg.stack_offset), arg.size)
+                            ml_def = Definition(ml_atom, ExternalCodeLocation(),
+                                                tags={ParameterTag(function=func_addr)})
+                            ml = self.annotate_with_def(self.top(self.arch.bits), ml_def)
+                            stack_address = self.get_stack_address(self.stack_address(arg.stack_offset))
+                            self.stack_definitions.store(stack_address, ml,
+                                                         endness=self.arch.memory_endness)
+                        else:
+                            raise TypeError('Unsupported parameter type %s.' % type(arg).__name__)
 
         # architecture dependent initialization
         if self.arch.name.startswith("PPC64"):
@@ -401,17 +406,17 @@ class ReachingDefinitionsState:
 
         return mv
 
-    def add_use(self, atom: Atom, code_loc) -> None:
+    def add_use(self, atom: Atom, code_loc: CodeLocation, expr: Optional[Any]=None) -> None:
         self._cycle(code_loc)
         self.codeloc_uses.update(self.get_definitions(atom))
 
-        self.live_definitions.add_use(atom, code_loc)
+        self.live_definitions.add_use(atom, code_loc, expr=expr)
 
-    def add_use_by_def(self, definition: Definition, code_loc: CodeLocation) -> None:
+    def add_use_by_def(self, definition: Definition, code_loc: CodeLocation, expr: Optional[Any]=None) -> None:
         self._cycle(code_loc)
         self.codeloc_uses.add(definition)
 
-        self.live_definitions.add_use_by_def(definition, code_loc)
+        self.live_definitions.add_use_by_def(definition, code_loc, expr=expr)
 
     def get_definitions(self, atom: Atom) -> Iterable[Definition]:
         yield from self.live_definitions.get_definitions(atom)
