@@ -1,6 +1,6 @@
 from typing import List, Set, Optional, Dict, Union, TYPE_CHECKING
 
-from ...sim_type import SimStruct
+from ...sim_type import SimStruct, SimTypePointer, SimTypeArray
 from ..analysis import Analysis, AnalysesHub
 from .simple_solver import SimpleSolver
 from .translator import TypeTranslator
@@ -26,7 +26,8 @@ class Typehoon(Analysis):
 
     User may specify ground truth, which will override all types at certain program points during constraint solving.
     """
-    def __init__(self, constraints, ground_truth=None, var_mapping: Optional[Dict['SimVariable','TypeVariable']]=None,
+    def __init__(self, constraints, ground_truth=None,
+                 var_mapping: Optional[Dict['SimVariable',Set['TypeVariable']]]=None,
                  prioritize_char_array_over_struct: bool=True,
                  must_struct: Optional[Set['TypeVariable']]=None,
                  ):
@@ -63,16 +64,23 @@ class Typehoon(Analysis):
     # Public methods
     #
 
-    def update_variable_types(self, func_addr: Union[int,str], var_to_typevar):
+    def update_variable_types(self, func_addr: Union[int,str], var_to_typevars):
 
-        for var, typevar in var_to_typevar.items():
-            type_ = self.simtypes_solution.get(typevar, None)
-            if type_ is not None:
-                # print("{} -> {}: {}".format(var, typevar, type_))
-                name = None
-                if isinstance(type_, SimStruct):
-                    name = type_.name
-                self.kb.variables[func_addr].set_variable_type(var, type_, name=name)
+        for var, typevars in var_to_typevars.items():
+            for typevar in typevars:
+                type_ = self.simtypes_solution.get(typevar, None)
+                if type_ is not None:
+                    # print("{} -> {}: {}".format(var, typevar, type_))
+                    # Hack: if a global address is of a pointer type and it is not an array, we unpack the type
+                    if func_addr == "global" and isinstance(type_, SimTypePointer) \
+                            and not isinstance(type_.pts_to, SimTypeArray):
+                        type_ = type_.pts_to
+
+                    name = None
+                    if isinstance(type_, SimStruct):
+                        name = type_.name
+
+                    self.kb.variables[func_addr].set_variable_type(var, type_, name=name)
 
     def pp_constraints(self) -> None:
         """
@@ -81,7 +89,11 @@ class Typehoon(Analysis):
         if self._var_mapping is None:
             raise ValueError("Variable mapping does not exist.")
 
-        typevar_to_var = dict((v, k) for k, v in self._var_mapping.items())
+        typevar_to_var = { }
+        for k, typevars in self._var_mapping.items():
+            for tv in typevars:
+                typevar_to_var[tv] = k
+
         print("### {} constraints".format(len(self._constraints)))
         for constraint in self._constraints:
             print("    " + constraint.pp_str(typevar_to_var))
@@ -96,7 +108,11 @@ class Typehoon(Analysis):
         if self.solution is None:
             raise RuntimeError("Please run type solver before calling pp_solution().")
 
-        typevar_to_var = dict((v, k) for k, v in self._var_mapping.items())
+        typevar_to_var = { }
+        for k, typevars in self._var_mapping.items():
+            for tv in typevars:
+                typevar_to_var[tv] = k
+
         print("### {} solutions".format(len(self.solution)))
         for typevar in sorted(self.solution.keys(), key=str):
             sol = self.solution[typevar]
@@ -168,7 +184,10 @@ class Typehoon(Analysis):
             # are all fields the same?
             if len(tc.fields) > 1 and all(tc.fields[off] == field0 for off in offsets):
                 # are all fields aligned properly?
-                alignment = field0.size
+                try:
+                    alignment = field0.size
+                except NotImplementedError:
+                    alignment = 1
                 if all(off % alignment == 0 for off in offsets):
                     # yeah!
                     max_offset = offsets[-1]
