@@ -8,7 +8,7 @@ from archinfo import RegisterName
 
 from .sim_type import SimType, SimTypeChar, SimTypePointer, SimTypeFixedSizeArray, SimTypeArray, SimTypeString, \
     SimTypeFunction, SimTypeFloat, SimTypeDouble, SimTypeReg, SimStruct, SimStructValue, SimTypeInt, SimTypeNum, \
-    SimUnion, SimTypeBottom, parse_signature
+    SimUnion, SimTypeBottom, parse_signature, SimTypeReference
 
 from .state_plugins.sim_action_object import SimActionObject
 
@@ -92,10 +92,15 @@ class AllocHelper:
         raise TypeError(type(val))
 
 
-def refine_locs_with_struct_type(arch: archinfo.Arch, locs: List, arg_type: SimType, offset: int=0):
+def refine_locs_with_struct_type(arch: archinfo.Arch, locs: List, arg_type: SimType, offset: int=0,
+                                 treat_bot_as_int=True):
     # CONTRACT FOR USING THIS METHOD: locs must be a list of locs which are all wordsize
     # ADDITIONAL NUANCE: this will not respect the need for big-endian integers to be stored at the end of words.
     # that's why this is named with_struct_type, because it will blindly trust the offsets given to it.
+
+    if treat_bot_as_int and isinstance(arg_type, SimTypeBottom):
+        arg_type = SimTypeInt(label=arg_type.label).with_arch(arch)
+
     if isinstance(arg_type, (SimTypeReg, SimTypeNum, SimTypeFloat)):
         seen_bytes = 0
         pieces = []
@@ -626,7 +631,9 @@ class SimCC:
         """
         return self.RETURN_ADDR
 
-    def next_arg(self, session, arg_type):
+    def next_arg(self, session: ArgSession, arg_type: SimType):
+        if isinstance(arg_type, (SimTypeArray, SimTypeFixedSizeArray)): # hack
+            arg_type = SimTypePointer(arg_type.elem_type).with_arch(self.arch)
         if isinstance(arg_type, (SimStruct, SimUnion, SimTypeFixedSizeArray)):
             raise TypeError(f"{self} doesn't know how to store aggregate types. Consider overriding next_arg to "
                             "implement its ABI logic")
@@ -870,8 +877,8 @@ class SimCC:
         if isinstance(arg, SimActionObject):
             return SimCC._standardize_value(arg.ast, ty, state, alloc)
         elif isinstance(arg, PointerWrapper):
-            if not isinstance(ty, SimTypePointer):
-                raise TypeError("Type mismatch: expected %s, got pointer-wrapper" % ty.name)
+            if not isinstance(ty, (SimTypePointer, SimTypeReference)):
+                raise TypeError("Type mismatch: expected %s, got pointer-wrapper" % ty)
 
             if arg.buffer:
                 if isinstance(arg.value, claripy.Bits):
@@ -882,7 +889,11 @@ class SimCC:
                     raise TypeError("PointerWrapper(buffer=True) can only be used with a bitvector or a bytestring")
             else:
                 child_type = SimTypeArray(ty.pts_to) if type(arg.value) in (str, bytes, list) else ty.pts_to
-                real_value = SimCC._standardize_value(arg.value, child_type, state, alloc)
+                try:
+                    real_value = SimCC._standardize_value(arg.value, child_type, state, alloc)
+                except TypeError as e: # this is a dangerous catch...
+                    raise TypeError(f"Failed to store pointer-wrapped data ({e.args[0]}). "
+                                    "Do you want a PointerWrapper(buffer=True)?") from None
             return alloc(real_value, state)
 
         elif isinstance(arg, (str, bytes)):
@@ -911,14 +922,14 @@ class SimCC:
                     raise TypeError("String %s is too long for %s" % (repr(arg), ty))
                 arg = arg.ljust(ty.length + 1, b'\0')
             else:
-                raise TypeError("Type mismatch: Expected %s, got char*" % ty.name)
+                raise TypeError("Type mismatch: Expected %s, got char*" % ty)
             val = SimCC._standardize_value(list(arg), SimTypeFixedSizeArray(SimTypeChar(), len(arg)), state, alloc)
             if ref:
                 val = alloc(val, state)
             return val
 
         elif isinstance(arg, list):
-            if isinstance(ty, SimTypePointer):
+            if isinstance(ty, (SimTypePointer, SimTypeReference)):
                 ref = True
                 subty = ty.pts_to
             elif isinstance(ty, SimTypeFixedSizeArray):
@@ -933,7 +944,7 @@ class SimCC:
                     if len(arg) != ty.length:
                         raise TypeError("Array %s is the wrong length for %s" % (repr(arg), ty))
             else:
-                raise TypeError("Type mismatch: Expected %s, got char*" % ty.name)
+                raise TypeError("Type mismatch: Expected %s, got char*" % ty)
 
             val = [SimCC._standardize_value(sarg, subty, state, alloc) for sarg in arg]
             if ref:
@@ -942,7 +953,7 @@ class SimCC:
 
         elif isinstance(arg, (tuple, dict, SimStructValue)):
             if not isinstance(ty, SimStruct):
-                raise TypeError("Type mismatch: Expected %s, got %s (i.e. struct)" % (ty.name, type(arg)))
+                raise TypeError("Type mismatch: Expected %s, got %s (i.e. struct)" % (ty, type(arg)))
             if type(arg) is not SimStructValue:
                 if len(arg) != len(ty.fields):
                     raise TypeError("Wrong number of fields in struct, expected %d got %d" % (len(ty.fields), len(arg)))
@@ -1109,6 +1120,8 @@ class SimCCCdecl(SimCC):
     ARCH = archinfo.ArchX86
 
     def next_arg(self, session, arg_type):
+        if isinstance(arg_type, (SimTypeArray, SimTypeFixedSizeArray)): # hack
+            arg_type = SimTypePointer(arg_type.elem_type).with_arch(self.arch)
         locs_size = 0
         byte_size = arg_type.size // self.arch.byte_width
         locs = []
@@ -1179,6 +1192,8 @@ class SimCCMicrosoftAMD64(SimCC):
 
     ArgSession = MicrosoftAMD64ArgSession
     def next_arg(self, session, arg_type):
+        if isinstance(arg_type, (SimTypeArray, SimTypeFixedSizeArray)): # hack
+            arg_type = SimTypePointer(arg_type.elem_type).with_arch(self.arch)
         try:
             int_loc = next(session.int_iter)
             fp_loc = next(session.fp_iter)
@@ -1312,7 +1327,16 @@ class SimCCSystemVAMD64(SimCC):
         some_both_args = [next(both_iter) for _ in range(len(args))]
 
         for arg in args:
-            if arg not in all_fp_args and arg not in all_int_args and arg not in some_both_args:
+            ex_arg = arg
+            # attempt to coerce the argument into a form that might show up in these lists
+            if type(ex_arg) is SimRegArg:
+                regfile_offset = arch.registers[ex_arg.reg_name][0]
+                while regfile_offset not in arch.register_names:
+                    regfile_offset -= 1
+                ex_arg.reg_name = arch.register_names[regfile_offset]
+                ex_arg.reg_offset = 0
+
+            if ex_arg not in all_fp_args and ex_arg not in all_int_args and ex_arg not in some_both_args:
                 if isinstance(arg, SimStackArg) and arg.stack_offset == 0:
                     continue        # ignore return address?
                 return False
@@ -1322,6 +1346,8 @@ class SimCCSystemVAMD64(SimCC):
     # https://raw.githubusercontent.com/wiki/hjl-tools/x86-psABI/x86-64-psABI-1.0.pdf
     # section 3.2.3
     def next_arg(self, session, arg_type):
+        if isinstance(arg_type, (SimTypeArray, SimTypeFixedSizeArray)): # hack
+            arg_type = SimTypePointer(arg_type.elem_type).with_arch(self.arch)
         state = session.getstate()
         classification = self._classify(arg_type)
         try:
@@ -1389,8 +1415,12 @@ class SimCCSystemVAMD64(SimCC):
     def _classify(self, ty, chunksize=None):
         if chunksize is None:
             chunksize = self.arch.bytes
-        nchunks = (ty.size // self.arch.byte_width + chunksize - 1) // chunksize
-        if isinstance(ty, (SimTypeInt, SimTypeChar, SimTypePointer, SimTypeNum)):
+        # treat BOT as INTEGER
+        if isinstance(ty, SimTypeBottom):
+            nchunks = 1
+        else:
+            nchunks = (ty.size // self.arch.byte_width + chunksize - 1) // chunksize
+        if isinstance(ty, (SimTypeInt, SimTypeChar, SimTypePointer, SimTypeNum, SimTypeBottom, SimTypeReference)):
             return ['INTEGER'] * nchunks
         elif isinstance(ty, (SimTypeFloat,)):
             return ['SSE'] + ['SSEUP'] * (nchunks - 1)
@@ -1502,12 +1532,134 @@ class SimCCAMD64WindowsSyscall(SimCCSyscall):
 
 class SimCCARM(SimCC):
     ARG_REGS = [ 'r0', 'r1', 'r2', 'r3' ]
-    FP_ARG_REGS = []    # TODO: ???
+    FP_ARG_REGS = []    # regular arg regs are used as fp arg regs
     CALLER_SAVED_REGS = []
     RETURN_ADDR = SimRegArg('lr', 4)
-    RETURN_VAL = SimRegArg('r0', 4)
+    RETURN_VAL = SimRegArg('r0', 4) #TODO Return val can also include reg r1
     ARCH = archinfo.ArchARM
 
+    # https://github.com/ARM-software/abi-aa/blob/60a8eb8c55e999d74dac5e368fc9d7e36e38dda4/aapcs32/aapcs32.rst#parameter-passing
+    def next_arg(self, session, arg_type):
+        if isinstance(arg_type, (SimTypeArray, SimTypeFixedSizeArray)):  # hack
+            arg_type = SimTypePointer(arg_type.elem_type).with_arch(self.arch)
+        state = session.getstate()
+        classification = self._classify(arg_type)
+        try:
+            mapped_classes = []
+            for idx, cls in enumerate(classification):
+                if cls == 'DOUBLEP':
+                    if session.getstate()[1] % 2 == 1: #doubles must start on an even register
+                        next(session.int_iter)
+
+                    if session.getstate()[1] == len(self.ARG_REGS) - 2:
+                        mapped_classes.append(next(session.int_iter))
+                        mapped_classes.append(next(session.both_iter))
+                    else:
+                        try:
+                            mapped_classes.append(next(session.int_iter))
+                            mapped_classes.append(next(session.int_iter))
+                        except StopIteration:
+                            mapped_classes.append(next(session.both_iter))
+                            mapped_classes.append(next(session.both_iter))
+                elif cls == 'NO_CLASS':
+                    raise NotImplementedError("Bug. Report to @rhelmot")
+                elif cls == 'MEMORY':
+                    mapped_classes.append(next(session.both_iter))
+                elif cls == 'INTEGER':
+                    try:
+                        mapped_classes.append(next(session.int_iter))
+                    except StopIteration:
+                        mapped_classes.append(next(session.both_iter))
+                elif cls == 'SINGLEP':
+                    try:
+                        mapped_classes.append(next(session.int_iter))
+                    except StopIteration:
+                        mapped_classes.append(next(session.both_iter))
+                else:
+                    raise NotImplementedError("Bug. Report to @rhelmot")
+        except StopIteration:
+            session.setstate(state)
+            mapped_classes = [next(session.both_iter) for _ in classification]
+
+        return refine_locs_with_struct_type(self.arch, mapped_classes, arg_type)
+
+    def _classify(self, ty, chunksize=None):
+        if chunksize is None:
+            chunksize = self.arch.bytes
+        # treat BOT as INTEGER
+        if isinstance(ty, SimTypeBottom):
+            nchunks = 1
+        else:
+            nchunks = (ty.size // self.arch.byte_width + chunksize - 1) // chunksize
+        if isinstance(ty, (SimTypeInt, SimTypeChar, SimTypePointer, SimTypeNum, SimTypeBottom, SimTypeReference)):
+            return ['INTEGER'] * nchunks
+        elif isinstance(ty, (SimTypeFloat,)):
+            if ty.size == 64:
+                return ['DOUBLEP']
+            elif ty.size == 32:
+                return ['SINGLEP']
+            return ['NO_CLASS']
+        elif isinstance(ty, (SimStruct, SimTypeFixedSizeArray, SimUnion)):
+            flattened = self._flatten(ty)
+            if flattened is None:
+                return ['MEMORY'] * nchunks
+            result = ['NO_CLASS'] * nchunks
+            for offset, subty_list in flattened.items():
+                for subty in subty_list:
+                    # is the smaller chunk size necessary? Genuinely unsure
+                    subresult = self._classify(subty, chunksize=1)
+                    idx_start = offset // chunksize
+                    idx_end = (offset + (subty.size // self.arch.byte_width) - 1) // chunksize
+                    for i, idx in enumerate(range(idx_start, idx_end + 1)):
+                        subclass = subresult[i * chunksize]
+                        result[idx] = self._combine_classes(result[idx], subclass)
+            return result
+        else:
+            raise NotImplementedError("Ummmmm... not sure what goes here. report bug to @rhelmot")
+
+    def _combine_classes(self, cls1, cls2):
+        if cls1 == cls2:
+            return cls1
+        if cls1 == 'NO_CLASS':
+            return cls2
+        if cls2 == 'NO_CLASS':
+            return cls1
+        if cls1 == 'MEMORY' or cls2 == 'MEMORY':
+            return 'MEMORY'
+        if cls1 == 'INTEGER' or cls2 == 'INTEGER':
+            return 'INTEGER'
+        return 'SSE'
+
+    def _flatten(self, ty) -> Optional[Dict[int, List[SimType]]]:
+        result: Dict[int, List[SimType]] = defaultdict(list)
+        if isinstance(ty, SimStruct):
+            if ty.packed:
+                return None
+            for field, subty in ty.fields.items():
+                offset = ty.offsets[field]
+                subresult = self._flatten(subty)
+                if subresult is None:
+                    return None
+                for suboffset, subsubty_list in subresult.items():
+                    result[offset + suboffset] += subsubty_list
+        elif isinstance(ty, SimTypeFixedSizeArray):
+            subresult = self._flatten(ty.elem_type)
+            if subresult is None:
+                return None
+            for suboffset, subsubty_list in subresult.items():
+                for idx in range(ty.length):
+                    # TODO I think we need an explicit stride field on array types
+                    result[idx * ty.elem_type.size // self.arch.byte_width + suboffset] += subsubty_list
+        elif isinstance(ty, SimUnion):
+            for field, subty in ty.members.items():
+                subresult = self._flatten(subty)
+                if subresult is None:
+                    return None
+                for suboffset, subsubty_list in subresult.items():
+                    result[suboffset] += subsubty_list
+        else:
+            result[0].append(ty)
+        return result
 
 class SimCCARMLinuxSyscall(SimCCSyscall):
     # TODO: Make sure all the information is correct
@@ -1567,16 +1719,135 @@ class SimCCAArch64LinuxSyscall(SimCCSyscall):
     def syscall_num(state):
         return state.regs.x8
 
-
 class SimCCO32(SimCC):
     ARG_REGS = [ 'a0', 'a1', 'a2', 'a3' ]
-    FP_ARG_REGS = []    # TODO: ???
+    FP_ARG_REGS = ['f12', 'f13', 'f14', 'f15']    # Note double precision args are split between f12-f13, f14-f15 and single precision only use f12 and f14
     STACKARG_SP_BUFF = 16
-    CALLER_SAVED_REGS = []   # TODO: ???
-    RETURN_ADDR = SimRegArg('lr', 4)
+    CALLER_SAVED_REGS = ['t9', 'gp']
+    RETURN_ADDR = SimRegArg('ra', 4)
     RETURN_VAL = SimRegArg('v0', 4)
     ARCH = archinfo.ArchMIPS32
 
+    # http://math-atlas.sourceforge.net/devel/assembly/mipsabi32.pdf Section 3-17
+    def next_arg(self, session, arg_type):
+        if isinstance(arg_type, (SimTypeArray, SimTypeFixedSizeArray)): # hack
+            arg_type = SimTypePointer(arg_type.elem_type).with_arch(self.arch)
+        state = session.getstate()
+        classification = self._classify(arg_type)
+        try:
+            mapped_classes = []
+            can_use_fp = True
+            for idx, cls in enumerate(classification):
+                if cls == 'DOUBLEP':
+                    mapped_classes.append(next(session.fp_iter))
+                    mapped_classes.append(next(session.fp_iter))
+                    if isinstance(arg_type, SimStruct) and idx < 2 and can_use_fp:
+                        next(session.fp_iter)  # consume next two fp regs since it's double precision
+                        next(session.fp_iter)
+                elif cls == 'NO_CLASS':
+                    raise NotImplementedError("Bug. Report to @rhelmot")
+                elif cls == 'MEMORY':
+                    mapped_classes.append(next(session.both_iter))
+                    can_use_fp = False
+                elif cls == 'INTEGER':
+                    mapped_classes.append(next(session.int_iter))
+                    can_use_fp = False
+                elif cls == 'SINGLEP':
+                    if isinstance(arg_type, SimStruct):
+                        if idx < 2 and can_use_fp:
+                            mapped_classes.append(next(session.fp_iter))
+                            next(session.int_iter) #Need to take up the arg slot
+                        else:
+                            mapped_classes.append(next(session.both_iter))
+                    else:
+                        mapped_classes.append(next(session.fp_iter))
+                        next(session.fp_iter) #consume f13 or f15 since it's single precision
+
+                else:
+                    raise NotImplementedError("Bug. Report to @rhelmot")
+        except StopIteration:
+            session.setstate(state)
+            mapped_classes = [next(session.both_iter) for _ in classification]
+
+        return refine_locs_with_struct_type(self.arch, mapped_classes, arg_type)
+
+    def _classify(self, ty, chunksize=None):
+        if chunksize is None:
+            chunksize = self.arch.bytes
+        # treat BOT as INTEGER
+        if isinstance(ty, SimTypeBottom):
+            nchunks = 1
+        else:
+            nchunks = (ty.size // self.arch.byte_width + chunksize - 1) // chunksize
+        if isinstance(ty, (SimTypeInt, SimTypeChar, SimTypePointer, SimTypeNum, SimTypeBottom, SimTypeReference)):
+            return ['INTEGER'] * nchunks
+        elif isinstance(ty, (SimTypeFloat,)):
+            if ty.size == 64:
+                return ['DOUBLEP']
+            elif ty.size == 32:
+                return ['SINGLEP']
+            return ['NO_CLASS']
+        elif isinstance(ty, (SimStruct, SimTypeFixedSizeArray, SimUnion)):
+            flattened = self._flatten(ty)
+            if flattened is None:
+                return ['MEMORY'] * nchunks
+            result = ['NO_CLASS'] * nchunks
+            for offset, subty_list in flattened.items():
+                for subty in subty_list:
+                    # is the smaller chunk size necessary? Genuinely unsure
+                    subresult = self._classify(subty, chunksize=1)
+                    idx_start = offset // chunksize
+                    idx_end = (offset + (subty.size // self.arch.byte_width) - 1) // chunksize
+                    for i, idx in enumerate(range(idx_start, idx_end + 1)):
+                        subclass = subresult[i * chunksize]
+                        result[idx] = self._combine_classes(result[idx], subclass)
+            return result
+        else:
+            raise NotImplementedError("Ummmmm... not sure what goes here. report bug to @rhelmot")
+
+    def _combine_classes(self, cls1, cls2):
+        if cls1 == cls2:
+            return cls1
+        if cls1 == 'NO_CLASS':
+            return cls2
+        if cls2 == 'NO_CLASS':
+            return cls1
+        if cls1 == 'MEMORY' or cls2 == 'MEMORY':
+            return 'MEMORY'
+        if cls1 == 'INTEGER' or cls2 == 'INTEGER':
+            return 'INTEGER'
+        return 'SSE'
+
+    def _flatten(self, ty) -> Optional[Dict[int, List[SimType]]]:
+        result: Dict[int, List[SimType]] = defaultdict(list)
+        if isinstance(ty, SimStruct):
+            if ty.packed:
+                return None
+            for field, subty in ty.fields.items():
+                offset = ty.offsets[field]
+                subresult = self._flatten(subty)
+                if subresult is None:
+                    return None
+                for suboffset, subsubty_list in subresult.items():
+                    result[offset + suboffset] += subsubty_list
+        elif isinstance(ty, SimTypeFixedSizeArray):
+            subresult = self._flatten(ty.elem_type)
+            if subresult is None:
+                return None
+            for suboffset, subsubty_list in subresult.items():
+                for idx in range(ty.length):
+                    # TODO I think we need an explicit stride field on array types
+                    result[idx * ty.elem_type.size // self.arch.byte_width + suboffset] += subsubty_list
+        elif isinstance(ty, SimUnion):
+            for field, subty in ty.members.items():
+                subresult = self._flatten(subty)
+                if subresult is None:
+                    return None
+                for suboffset, subsubty_list in subresult.items():
+                    result[suboffset] += subsubty_list
+        else:
+            result[0].append(ty)
+        return result
 
 class SimCCO32LinuxSyscall(SimCCSyscall):
     # TODO: Make sure all the information is correct
@@ -1601,9 +1872,10 @@ class SimCCO32LinuxSyscall(SimCCSyscall):
 
 class SimCCO64(SimCC):      # TODO: add n32 and n64
     ARG_REGS = [ 'a0', 'a1', 'a2', 'a3' ]
+    CALLER_SAVED_REGS = ['t9', 'gp']
     FP_ARG_REGS = []    # TODO: ???
     STACKARG_SP_BUFF = 32
-    RETURN_ADDR = SimRegArg('lr', 8)
+    RETURN_ADDR = SimRegArg('ra', 8)
     RETURN_VAL = SimRegArg('v0', 8)
     ARCH = archinfo.ArchMIPS64
 
