@@ -79,7 +79,7 @@ class SimEngineLight(
 
         # local variables
         self.state = None
-        self.arch = None
+        self.arch: archinfo.Arch = None
         self.block = None
         self._call_stack = None
 
@@ -993,7 +993,21 @@ class SimEngineLightAILMixin(SimEngineLightMixin):
             return ailment.Expr.BinaryOp(expr.idx, 'Div', [expr_0, expr_1], expr.signed, **expr.tags)
 
     def _ail_handle_DivMod(self, expr):
-        return self._ail_handle_Div(expr)
+
+        arg0, arg1 = expr.operands
+
+        expr_0 = self._expr(arg0)
+        expr_1 = self._expr(arg1)
+
+        if expr_0 is None:
+            expr_0 = arg0
+        if expr_1 is None:
+            expr_1 = arg1
+
+        try:
+            return expr_0 % expr_1
+        except TypeError:
+            return ailment.Expr.BinaryOp(expr.idx, 'DivMod', [expr_0, expr_1], expr.signed, **expr.tags)
 
     def _ail_handle_Mul(self, expr):
 
