@@ -9,7 +9,9 @@ from ... import sim_options as o
 from ...errors import SimEngineError, SimTranslationError
 from cle import CLEError
 from ...state_plugins.inspect import BP_AFTER, BP_BEFORE
-from ..engine import SimEngine
+from ...sim_type import SimTypeNum, SimTypeFunction, parse_type
+from ..engine import SuccessorsMixin
+from ..procedure import ProcedureMixin
 from .exceptions import BlockTerminationNotice, IncorrectLocationException
 from .statements import (SimSootStmt_Return, SimSootStmt_ReturnVoid,
                          translate_stmt)
@@ -19,16 +21,12 @@ l = logging.getLogger('angr.engines.soot.engine')
 
 # pylint: disable=arguments-differ
 
-class SimEngineSoot(SimEngine):
+class SootMixin(SuccessorsMixin, ProcedureMixin):
     """
     Execution engine based on Soot.
     """
 
-    def __init__(self, project=None, **kwargs):
-        super(SimEngineSoot, self).__init__(**kwargs)
-        self.project = project
-
-    def lift(self, addr=None, the_binary=None, **kwargs):
+    def lift_soot(self, addr=None, the_binary=None, **kwargs): # pylint: disable=unused-argument, no-self-use
         assert isinstance(addr, SootAddressDescriptor)
 
         method, stmt_idx = addr.method, addr.stmt_idx
@@ -61,10 +59,10 @@ class SimEngineSoot(SimEngine):
                     return block
             return None
 
-    def _check(self, state, *args, **kwargs):
-        return isinstance(state._ip, SootAddressDescriptor)
-
-    def _process(self, state, successors, *args, **kwargs):
+    def process_successors(self, successors, **kwargs):
+        state = self.state
+        if not isinstance(state._ip, SootAddressDescriptor):
+            return super().process_successors(successors, **kwargs)
         addr = state._ip
 
         if isinstance(addr, SootAddressTerminator):
@@ -74,24 +72,21 @@ class SimEngineSoot(SimEngine):
         if self.project.use_sim_procedures:
             procedure = self._get_sim_procedure(addr)
             if procedure is not None:
-                self.project.factory.procedure_engine._process(state, successors, procedure)
+                self.process_procedure(state, successors, procedure)
                 return
 
-        # skip android system methods (-> use sim procedures)
-        if addr.method.class_name.startswith('android'):
-            self._add_return_exit(state, successors)
-            return
-
         binary = state.regs._ip_binary
         method = binary.get_soot_method(addr.method, none_if_missing=True)
-        if not method:
+
+        # TODO make the skipping of code in "android.*" classes optional
+        if addr.method.class_name.startswith('android.') or not method:
             # This means we are executing code that is not in CLE, typically library code.
             # We may want soot -> pysoot -> cle to export at least the method names of the libraries
             # (soot has a way to deal with this), as of now we just "simulate" a return.
             # Note: If we have a sim procedure, we should not reach this point.
             l.warning("Try to execute non-loaded code %s. Execute unconstrained SimProcedure.", addr)
             # STEP 1: Get unconstrained SimProcedure
-            procedure = self.get_unconstrained_simprocedure(addr)
+            procedure = self.get_unconstrained_simprocedure()
             # STEP 2: Pass Method descriptor as Parameter
 
             # check if there are already params in the stack
@@ -104,7 +99,7 @@ class SimEngineSoot(SimEngine):
             # store all function arguments in memory, starting from the last param index
             state.memory.store(SimSootValue_ParamRef(param_idx, None), addr.method)
             # STEP 4: Execute unconstrained procedure
-            self.project.factory.procedure_engine._process(state, successors, procedure)
+            self.process_procedure(state, successors, procedure)
             # self._add_return_exit(state, successors)
             return
 
@@ -115,15 +110,16 @@ class SimEngineSoot(SimEngine):
         else:
             # l.debug("Continue executing block %s", addr)
             l.debug("Continue executing block %s \n\n%s\n", addr, block)
-        self._handle_block(state, successors, block, starting_stmt_idx, method)
+        self._handle_soot_block(state, successors, block, starting_stmt_idx, method)
 
         successors.processed = True
 
-    def _handle_block(self, state, successors, block, starting_stmt_idx, method=None):
+    def _handle_soot_block(self, state, successors, block, starting_stmt_idx, method=None):
+        stmt = stmt_idx = None
         for tindex, stmt in enumerate(block.statements[starting_stmt_idx:]):
             stmt_idx = starting_stmt_idx + tindex
             state._inspect('statement', BP_BEFORE, statement=stmt_idx)
-            terminate = self._handle_statement(state, successors, stmt_idx, stmt)
+            terminate = self._handle_soot_stmt(state, successors, stmt_idx, stmt)
             state._inspect('statement', BP_AFTER)
             if terminate:
                 break
@@ -137,14 +133,14 @@ class SimEngineSoot(SimEngine):
                 if next_addr is not None:
                     successors.add_successor(state.copy(), next_addr, state.solver.true, 'Ijk_Boring')
 
-    def _handle_statement(self, state, successors, stmt_idx, stmt):
+    def _handle_soot_stmt(self, state, successors, stmt_idx, stmt):
         # execute statement
         try:
             l.debug("Executing statement: %s", stmt)
             s_stmt = translate_stmt(stmt, state)
         except SimEngineError as e:
             l.error("Skipping statement: %s", e)
-            return
+            return False
 
         # add invoke exit
         if s_stmt.has_invoke_target:
@@ -226,7 +222,7 @@ class SimEngineSoot(SimEngine):
 
         return proc
 
-    def get_unconstrained_simprocedure(self, addr):
+    def get_unconstrained_simprocedure(self):
         # Delayed import
         from ...procedures import SIM_PROCEDURES
 
@@ -277,7 +273,7 @@ class SimEngineSoot(SimEngine):
     @staticmethod
     def setup_arguments(state, args):
         # if available, store the 'this' reference
-        if args[0].is_this_ref:
+        if len(args) > 0 and args[0].is_this_ref:
             this_ref = args.pop(0)
             local = SimSootValue_Local('this', this_ref.type)
             state.javavm_memory.store(local, this_ref.value)
@@ -355,7 +351,9 @@ class SimEngineSoot(SimEngine):
         if ret_var is not None:
             # get return symbol from native state
             native_cc = javavm_simos.get_native_cc()
-            ret_symbol = native_cc.get_return_val(native_state).to_claripy()
+            ret_symbol = native_cc.return_val(
+                javavm_simos.get_native_type(ret_var.type)
+            ).get_value(native_state).to_claripy()
             # convert value to java type
             if ret_var.type in ArchSoot.primitive_types:
                 # return value has a primitive type
@@ -373,7 +371,7 @@ class SimEngineSoot(SimEngine):
             ret_value = None
 
         # teardown return state
-        SimEngineSoot.prepare_return_state(ret_state, ret_value)
+        SootMixin.prepare_return_state(ret_state, ret_value)
 
         # finally, delete all local references
         ret_state.jni_references.clear_local_references()
@@ -394,8 +392,7 @@ class SimEngineSoot(SimEngine):
         if args and args[0].is_this_ref:
             # instance method call
             # => pass 'this' reference to native code
-            ref = this_ref = args.pop(0)
-            args.insert(1, this_ref)
+            ref = args.pop(0)
         else:
             # static method call
             # => pass 'class' reference to native code
@@ -403,9 +400,16 @@ class SimEngineSoot(SimEngine):
             ref = SootArgument(class_, "Class")
 
         # add to args
-        args = [jni_env, ref] + args
+        final_args = [jni_env, ref] + args
+
+        # Step 3: generate C prototype from java_method
+        voidp = parse_type('void*')
+        arg_types = [voidp, voidp] + [state.project.simos.get_native_type(ty) for ty in java_method.params]
+        ret_type = state.project.simos.get_native_type(java_method.ret)
+        prototype = SimTypeFunction(args=arg_types, returnty=ret_type)
 
         # Step 3: create native invoke state
-        return state.project.simos.state_call(native_addr, *args,
+        return state.project.simos.state_call(native_addr, *final_args,
                                               base_state=state,
+                                              prototype=prototype,
                                               ret_type=java_method.ret)
