@@ -3,7 +3,7 @@ import claripy
 import logging
 import time
 
-from angr import sim_options as o
+from angr import sim_options as o, errors
 
 l = logging.getLogger(name=__name__)
 
@@ -41,6 +41,13 @@ x86g_dirtyhelper_RDTSC = amd64g_dirtyhelper_RDTSC
 
 # Copied basically directly from the vex source
 def amd64g_dirtyhelper_CPUID_baseline(state, _):
+    if 'CPUID_SYMBOLIC' in state.options:
+        state.regs.rax = claripy.BVS("cpuid_a", 32).zero_extend(32)
+        state.regs.rbx = claripy.BVS("cpuid_b", 32).zero_extend(32)
+        state.regs.rcx = claripy.BVS("cpuid_c", 32).zero_extend(32)
+        state.regs.rdx = claripy.BVS("cpuid_d", 32).zero_extend(32)
+        return None, []
+
     old_eax = state.regs.rax[31:0]
     def SET_ABCD(a, b, c, d, condition=None):
         if condition is None:
@@ -142,6 +149,13 @@ def amd64g_dirtyhelper_XRSTOR_COMPONENT_1_EXCLUDING_XMMREGS(state, _, addr):
 
 
 def CORRECT_amd64g_dirtyhelper_CPUID_avx_and_cx16(state, _):
+    if 'CPUID_SYMBOLIC' in state.options:
+        state.regs.rax = claripy.BVS("cpuid_a", 32).zero_extend(32)
+        state.regs.rbx = claripy.BVS("cpuid_b", 32).zero_extend(32)
+        state.regs.rcx = claripy.BVS("cpuid_c", 32).zero_extend(32)
+        state.regs.rdx = claripy.BVS("cpuid_d", 32).zero_extend(32)
+        return None, []
+
     old_eax = state.regs.rax[31:0]
     old_ecx = state.regs.rcx[31:0]
 
@@ -220,6 +234,13 @@ def amd64g_dirtyhelper_SxDT(state, addr, op): #pylint:disable=unused-argument
     return None, [ ]
 
 def x86g_dirtyhelper_CPUID_sse0(state, _):
+    if 'CPUID_SYMBOLIC' in state.options:
+        state.regs.eax = claripy.BVS("cpuid_a", 32)
+        state.regs.ebx = claripy.BVS("cpuid_b", 32)
+        state.regs.ecx = claripy.BVS("cpuid_c", 32)
+        state.regs.edx = claripy.BVS("cpuid_d", 32)
+        return None, []
+
     old_eax = state.regs.eax
 
     def SET_ABCD(a, b, c, d, condition=None, condition2=None):
@@ -245,6 +266,13 @@ x86g_dirtyhelper_CPUID_sse2 = x86g_dirtyhelper_CPUID_sse0
 x86g_dirtyhelper_CPUID_sse3 = x86g_dirtyhelper_CPUID_sse0
 
 def CORRECT_x86g_dirtyhelper_CPUID_sse2(state, _):
+    if 'CPUID_SYMBOLIC' in state.options:
+        state.regs.eax = claripy.BVS("cpuid_a", 32)
+        state.regs.ebx = claripy.BVS("cpuid_b", 32)
+        state.regs.ecx = claripy.BVS("cpuid_c", 32)
+        state.regs.edx = claripy.BVS("cpuid_d", 32)
+        return None, []
+
     old_eax = state.regs.eax
     old_ecx = state.regs.ecx
 
@@ -338,6 +366,8 @@ def x86g_dirtyhelper_FINIT(state, gsptr): #pylint:disable=unused-argument
     state.regs.ftop = 0
     return None, [ ]
 
+amd64g_dirtyhelper_FINIT = x86g_dirtyhelper_FINIT
+
 def x86g_dirtyhelper_write_cr0(state, value):
     # make a deep copy of the arch before modifying it so we don't accidentally modify it for all other states
     state.arch = state.arch.copy()
@@ -383,3 +413,22 @@ def x86g_dirtyhelper_storeF80le(state, addr, qword):
     assert len(tbyte) == 80
     state.memory.store(addr, tbyte, endness='Iend_LE')
     return None, []
+
+def x86g_dirtyhelper_RDMSR(state, msr):
+    try:
+        msr_conc = state.solver.eval_one(msr)
+    except errors.SimSolverError:
+        return state.solver.BVS('rdmsr_?', 64, key=('cpu', 'rdmsr', '?')), []
+    else:
+        return state.solver.BVS('rdmsr_%#x' % msr_conc, 64, key=('cpu', 'rdmsr', msr_conc), eternal=True), []
+
+def x86g_dirtyhelper_XGETBV(state, reg):
+    try:
+        reg_conc = state.solver.eval_one(reg)
+    except errors.SimSolverError:
+        return state.solver.BVS('xgetbv_?', 64, key=('cpu', 'xgetbv', '?')), []
+    else:
+        return state.solver.BVS('xgetbv_%#x' % reg_conc, 64, key=('cpu', 'xgetbv', reg_conc), eternal=True), []
+
+amd64g_dirtyhelper_RDMSR = x86g_dirtyhelper_RDMSR
+amd64g_dirtyhelper_XGETBV = x86g_dirtyhelper_XGETBV
