@@ -1,13 +1,13 @@
 import weakref
-from typing import Optional, Iterable, Dict, Set, Generator, Tuple, Union, TYPE_CHECKING
+from typing import Optional, Iterable, Dict, Set, Generator, Tuple, Union, Any, TYPE_CHECKING
 import logging
 
+from collections import defaultdict
+
 import claripy
 from claripy.annotation import Annotation
 import archinfo
 
-from collections import defaultdict
-
 from ...errors import SimMemoryMissingError, SimMemoryError
 from ...storage.memory_mixins import MultiValuedMemory
 from ...storage.memory_mixins.paged_memory.pages.multi_values import MultiValues
@@ -57,6 +57,8 @@ class DefinitionAnnotation(Annotation):
         else:
             raise ValueError("DefinitionAnnotation can only check equality with other DefinitionAnnotation")
 
+    def __repr__(self):
+        return f"<{self.__class__.__name__}({repr(self.definition)})"
 
 # pylint: disable=W1116
 class LiveDefinitions:
@@ -212,17 +214,27 @@ class LiveDefinitions:
         return "stack_base" in addr.variables
 
     @staticmethod
-    def get_stack_offset(addr: claripy.ast.Base) -> Optional[int]:
+    def get_stack_offset(addr: claripy.ast.Base, had_stack_base=False) -> Optional[int]:
+        if had_stack_base and addr.op == "BVV":
+            return addr._model_concrete.value
+        if "TOP" in addr.variables:
+            return None
         if "stack_base" in addr.variables:
             if addr.op == "BVS":
                 return 0
             elif addr.op == "__add__":
-                if len(addr.args) == 2 and addr.args[1].op == "BVV":
-                    return addr.args[1]._model_concrete.value
-                if len(addr.args) == 1:
+                if len(addr.args) == 2:
+                    off0 = LiveDefinitions.get_stack_offset(addr.args[0], had_stack_base=True)
+                    off1 = LiveDefinitions.get_stack_offset(addr.args[1], had_stack_base=True)
+                    if off0 is not None and off1 is not None:
+                        return off0 + off1
+                elif len(addr.args) == 1:
                     return 0
-            elif addr.op == "__sub__" and len(addr.args) == 2 and addr.args[1].op == "BVV":
-                return -addr.args[1]._model_concrete.value
+            elif addr.op == "__sub__" and len(addr.args) == 2:
+                off0 = LiveDefinitions.get_stack_offset(addr.args[0], had_stack_base=True)
+                off1 = LiveDefinitions.get_stack_offset(addr.args[1], had_stack_base=True)
+                if off0 is not None and off1 is not None:
+                    return off0 - off1
         return None
 
     @staticmethod
@@ -401,16 +413,16 @@ class LiveDefinitions:
 
         return d
 
-    def add_use(self, atom: Atom, code_loc: CodeLocation) -> None:
+    def add_use(self, atom: Atom, code_loc: CodeLocation, expr: Optional[Any]=None) -> None:
         if isinstance(atom, Register):
-            self._add_register_use(atom, code_loc)
+            self._add_register_use(atom, code_loc, expr=expr)
         elif isinstance(atom, MemoryLocation):
             if isinstance(atom.addr, SpOffset):
-                self._add_stack_use(atom, code_loc)
+                self._add_stack_use(atom, code_loc, expr=expr)
             elif isinstance(atom.addr, HeapAddress):
-                self._add_heap_use(atom, code_loc)
+                self._add_heap_use(atom, code_loc, expr=expr)
             elif isinstance(atom.addr, int):
-                self._add_memory_use(atom, code_loc)
+                self._add_memory_use(atom, code_loc, expr=expr)
             else:
                 # ignore RegisterOffset
                 pass
@@ -419,16 +431,16 @@ class LiveDefinitions:
         else:
             raise TypeError("Unsupported atom type %s." % type(atom))
 
-    def add_use_by_def(self, definition: Definition, code_loc: CodeLocation) -> None:
+    def add_use_by_def(self, definition: Definition, code_loc: CodeLocation, expr: Any=None) -> None:
         if isinstance(definition.atom, Register):
-            self._add_register_use_by_def(definition, code_loc)
+            self._add_register_use_by_def(definition, code_loc, expr=expr)
         elif isinstance(definition.atom, MemoryLocation):
             if isinstance(definition.atom.addr, SpOffset):
-                self._add_stack_use_by_def(definition, code_loc)
+                self._add_stack_use_by_def(definition, code_loc, expr=expr)
             elif isinstance(definition.atom.addr, HeapAddress):
-                self._add_heap_use_by_def(definition, code_loc)
+                self._add_heap_use_by_def(definition, code_loc, expr=expr)
             elif isinstance(definition.atom.addr, int):
-                self._add_memory_use_by_def(definition, code_loc)
+                self._add_memory_use_by_def(definition, code_loc, expr=expr)
             else:
                 # ignore RegisterOffset
                 pass
@@ -530,7 +542,7 @@ class LiveDefinitions:
     # Private methods
     #
 
-    def _add_register_use(self, atom: Register, code_loc: CodeLocation) -> None:
+    def _add_register_use(self, atom: Register, code_loc: CodeLocation, expr: Optional[Any]=None) -> None:
         # get all current definitions
         try:
             values: MultiValues = self.register_definitions.load(atom.reg_offset, size=atom.size)
@@ -540,25 +552,25 @@ class LiveDefinitions:
         for vs in values.values.values():
             for v in vs:
                 for def_ in self.extract_defs(v):
-                    self._add_register_use_by_def(def_, code_loc)
+                    self._add_register_use_by_def(def_, code_loc, expr=expr)
 
-    def _add_register_use_by_def(self, def_: Definition, code_loc: CodeLocation) -> None:
-        self.register_uses.add_use(def_, code_loc)
+    def _add_register_use_by_def(self, def_: Definition, code_loc: CodeLocation, expr: Optional[Any]=None) -> None:
+        self.register_uses.add_use(def_, code_loc, expr=expr)
         self.uses_by_codeloc[code_loc].add(def_)
 
-    def _add_stack_use(self, atom: MemoryLocation, code_loc: CodeLocation) -> None:
+    def _add_stack_use(self, atom: MemoryLocation, code_loc: CodeLocation, expr: Optional[Any]=None) -> None:
 
         if not isinstance(atom.addr, SpOffset):
             raise TypeError("Atom %r is not a stack location atom." % atom)
 
         for current_def in self.get_definitions(atom):
-            self._add_stack_use_by_def(current_def, code_loc)
+            self._add_stack_use_by_def(current_def, code_loc, expr=expr)
 
-    def _add_stack_use_by_def(self, def_: Definition, code_loc: CodeLocation) -> None:
-        self.stack_uses.add_use(def_, code_loc)
+    def _add_stack_use_by_def(self, def_: Definition, code_loc: CodeLocation, expr: Optional[Any]=None) -> None:
+        self.stack_uses.add_use(def_, code_loc, expr=expr)
         self.uses_by_codeloc[code_loc].add(def_)
 
-    def _add_heap_use(self, atom: MemoryLocation, code_loc: CodeLocation) -> None:
+    def _add_heap_use(self, atom: MemoryLocation, code_loc: CodeLocation, expr: Optional[Any]=None) -> None:
 
         if not isinstance(atom.addr, HeapAddress):
             raise TypeError("Atom %r is not a heap location atom." % atom)
@@ -566,22 +578,22 @@ class LiveDefinitions:
         current_defs = self.heap_definitions.get_objects_by_offset(atom.addr.value)
 
         for current_def in current_defs:
-            self._add_heap_use_by_def(current_def, code_loc)
+            self._add_heap_use_by_def(current_def, code_loc, expr=expr)
 
-    def _add_heap_use_by_def(self, def_: Definition, code_loc: CodeLocation) -> None:
-        self.heap_uses.add_use(def_, code_loc)
+    def _add_heap_use_by_def(self, def_: Definition, code_loc: CodeLocation, expr: Optional[Any]=None) -> None:
+        self.heap_uses.add_use(def_, code_loc, expr=expr)
         self.uses_by_codeloc[code_loc].add(def_)
 
-    def _add_memory_use(self, atom: MemoryLocation, code_loc: CodeLocation) -> None:
+    def _add_memory_use(self, atom: MemoryLocation, code_loc: CodeLocation, expr: Optional[Any]=None) -> None:
 
         # get all current definitions
         current_defs: Iterable[Definition] = self.get_definitions(atom)
 
         for current_def in current_defs:
-            self._add_memory_use_by_def(current_def, code_loc)
+            self._add_memory_use_by_def(current_def, code_loc, expr=expr)
 
-    def _add_memory_use_by_def(self, def_: Definition, code_loc: CodeLocation) -> None:
-        self.memory_uses.add_use(def_, code_loc)
+    def _add_memory_use_by_def(self, def_: Definition, code_loc: CodeLocation, expr: Optional[Any]=None) -> None:
+        self.memory_uses.add_use(def_, code_loc, expr=expr)
         self.uses_by_codeloc[code_loc].add(def_)
 
     def _add_tmp_use(self, atom: Tmp, code_loc: CodeLocation) -> None:
