@@ -14,7 +14,13 @@ class syscall(angr.SimProcedure):
         if self.successors is not None:
             self.successors.artifacts['resolves'] = resolves
 
-        return self.state.solver.Unconstrained("syscall_stub_%s" % self.display_name, self.state.arch.bits, key=('syscall', '?', self.display_name))
+        # code duplicated from ReturnUnconstrained
+        size = self.prototype.returnty.size
+        # ummmmm do we really want to rely on this behavior?
+        if size is NotImplemented:
+            return None
+        else:
+            return self.state.solver.Unconstrained("syscall_stub_%s" % self.display_name, size, key=('syscall', '?', self.display_name))
 
     def __repr__(self):
         if 'resolves' in self.kwargs:
