@@ -2,14 +2,20 @@ import sys
 import itertools
 import types
 from collections import defaultdict
+from typing import List, Tuple, DefaultDict
+import logging
 
 import claripy
 import mulpyplexer
 
 from .misc.hookset import HookSet
 from .misc.ux import once
+from .errors import SimError, SimMergeError
+from .sim_state import SimState
+from .state_hierarchy import StateHierarchy
+from .errors import AngrError, SimUnsatError, SimulationManagerError
+from .sim_options import LAZY_SOLVES
 
-import logging
 l = logging.getLogger(name=__name__)
 
 
@@ -49,13 +55,15 @@ class SimulationManager:
                             the state and some information about the error are placed in this list. You can adjust the
                             list of caught exceptions with the `resilience` parameter.
     :ivar stashes:          All the stashes on this instance, as a dictionary.
-    :ivar completion_mode:  A function describing how multiple exploration techniques with the ``complete`` hook set will
-                            interact. By default, the builtin function ``any``.
+    :ivar completion_mode:  A function describing how multiple exploration techniques with the ``complete`` hook set
+                            will interact. By default, the builtin function ``any``.
     """
     ALL = '_ALL'
     DROP = '_DROP'
 
-    _integral_stashes = 'active', 'stashed', 'pruned', 'unsat', 'errored', 'deadended', 'unconstrained'
+    _integral_stashes = (
+        'active', 'stashed', 'pruned', 'unsat', 'errored', 'deadended', 'unconstrained'
+    ) # type: Tuple[str]
 
     def __init__(self,
             project,
@@ -69,13 +77,15 @@ class SimulationManager:
             completion_mode=any,
             techniques=None,
             **kwargs):
-        super(SimulationManager, self).__init__()
+        super().__init__()
 
         self._project = project
         self.completion_mode = completion_mode
         self._errored = []
 
-        self._stashes = self._create_integral_stashes() if stashes is None else stashes
+        if stashes is None:
+            stashes = self._create_integral_stashes()
+        self._stashes = stashes # type: defaultdict[str, List['SimState']]
         self._hierarchy = StateHierarchy() if hierarchy is None else hierarchy
         self._save_unsat = save_unsat
         self._auto_drop = {SimulationManager.DROP, }
@@ -84,8 +94,10 @@ class SimulationManager:
         if resilience is None:
             self._resilience = (AngrError, SimError, claripy.ClaripyError)
         elif resilience is True:
-            self._resilience = (AngrError, SimError, claripy.ClaripyError, \
-                         KeyError, IndexError, TypeError, ValueError, ArithmeticError, MemoryError)
+            self._resilience = (
+                AngrError, SimError, claripy.ClaripyError, KeyError, IndexError, TypeError, ValueError, ArithmeticError,
+                MemoryError
+            )
         elif resilience is False:
             self._resilience = ()
         else:
@@ -98,6 +110,7 @@ class SimulationManager:
 
         if kwargs.pop('veritesting', False):
             self.use_technique(Veritesting(**kwargs.get('veritesting_options', {})))
+        kwargs.pop('veritesting_options', {})
 
         threads = kwargs.pop('threads', None)
         if threads is not None:
@@ -122,7 +135,10 @@ class SimulationManager:
 
     def __repr__(self):
         stashes_repr = ', '.join(("%d %s" % (len(v), k)) for k, v in self._stashes.items() if len(v) != 0)
-        return "<SimulationManager with %s%s>" % (stashes_repr if stashes_repr else 'all stashes empty', ' (%d errored)' % len(self.errored) if self.errored else '')
+        if not stashes_repr:
+            stashes_repr = 'all stashes empty'
+        errored_repr = ' (%d errored)' % len(self.errored) if self.errored else ''
+        return f"<SimulationManager with {stashes_repr}{errored_repr}>"
 
     def __getattr__(self, item):
         try:
@@ -131,14 +147,18 @@ class SimulationManager:
             return SimulationManager._fetch_states(self, stash=item)
 
     def __dir__(self):
-        return list(self.__dict__) + dir(type(self)) + list(self._stashes) + ['one_' + stash for stash in self._stashes] + ['mp_' + stash for stash in self._stashes]
+        return list(self.__dict__) + \
+               dir(type(self)) + \
+               list(self._stashes) + \
+               ['one_' + stash for stash in self._stashes] + \
+               ['mp_' + stash for stash in self._stashes]
 
     @property
     def errored(self):
         return self._errored
 
     @property
-    def stashes(self):
+    def stashes(self) -> DefaultDict[str, List['SimState']]:
         return self._stashes
 
     def mulpyplex(self, *stashes):
@@ -233,11 +253,30 @@ class SimulationManager:
         num_find += len(self._stashes[find_stash]) if find_stash in self._stashes else 0
         tech = self.use_technique(Explorer(find, avoid, find_stash, avoid_stash, cfg, num_find))
 
+        # Modify first Veritesting so that they can work together.
+        deviation_filter_saved = None
+        for t in self._techniques:
+            if isinstance(t,Veritesting):
+                deviation_filter_saved = t.options.get("deviation_filter",None)
+                if deviation_filter_saved is not None:
+                    t.options["deviation_filter"] = lambda s: tech.find(s) or tech.avoid(s) or deviation_filter_saved(s)
+                else:
+                    t.options["deviation_filter"] = lambda s: tech.find(s) or tech.avoid(s)
+                break
+
         try:
             self.run(stash=stash, n=n, **kwargs)
         finally:
             self.remove_technique(tech)
 
+        for t in self._techniques:
+            if isinstance(t,Veritesting):
+                if deviation_filter_saved is None:
+                    del t.options["deviation_filter"]
+                else:
+                    t.options["deviation_filter"] = deviation_filter_saved
+                break
+
         return self
 
     def run(self, stash='active', n=None, until=None, **kwargs):
@@ -340,10 +379,10 @@ class SimulationManager:
                 continue
 
             pre_errored = len(self._errored)
-            successors = self.step_state(state, successor_func=successor_func, **run_args)
 
+            successors = self.step_state(state, successor_func=successor_func, **run_args)
             # handle degenerate stepping cases here. desired behavior:
-            # if a step produced only unsat states, always add them to the unsat stash since this usually indicates a bug
+            # if a step produced only unsat states, always add them to the unsat stash since this usually indicates bugs
             # if a step produced sat states and save_unsat is False, drop the unsats
             # if a step produced no successors, period, add the original state to deadended
 
@@ -367,6 +406,9 @@ class SimulationManager:
 
         self._clear_states(stash=stash)
         for to_stash, states in bucket.items():
+            for state in states:
+                if self._hierarchy:
+                    self._hierarchy.add_state(state)
             self._store_states(to_stash or stash, states)
 
         if step_func is not None:
@@ -384,10 +426,15 @@ class SimulationManager:
                        'unconstrained': successors.unconstrained_successors}
 
         except (SimUnsatError, claripy.UnsatError) as e:
+            if LAZY_SOLVES not in state.options:
+                self._errored.append(ErrorRecord(state, e, sys.exc_info()[2]))
+                stashes = {}
+            else:
+                stashes = {'pruned': [state]}
+
             if self._hierarchy:
                 self._hierarchy.unreachable_state(state)
                 self._hierarchy.simplify()
-            stashes = {'pruned': [state]}
 
         except tuple(self._resilience) as e:
             self._errored.append(ErrorRecord(state, e, sys.exc_info()[2]))
@@ -607,7 +654,7 @@ class SimulationManager:
                 [x.func_addr for x in state.callstack],
                 set(state.posix.fd) if state.has_plugin('posix') else None)
 
-    def merge(self, merge_func=None, merge_key=None, stash='active'):
+    def merge(self, merge_func=None, merge_key=None, stash='active', prune=True):
         """
         Merge the states in a given stash.
 
@@ -617,14 +664,17 @@ class SimulationManager:
         :param merge_key:   If provided, should be a function that takes a state and returns a key that will compare
                             equal for all states that are allowed to be merged together, as a first aproximation.
                             By default: uses PC, callstack, and open file descriptors.
+        :param prune:       Whether to prune the stash prior to merging it
 
         :returns:           The simulation manager, for chaining.
         :rtype:             SimulationManager
         """
-        self.prune(from_stash=stash)
+        if prune:
+            self.prune(from_stash=stash)
         to_merge = self._fetch_states(stash=stash)
         not_to_merge = []
-        if merge_key is None: merge_key = self._merge_key
+        if merge_key is None:
+            merge_key = self._merge_key
 
         merge_groups = [ ]
         while to_merge:
@@ -654,7 +704,7 @@ class SimulationManager:
     def _store_states(self, stash, states):
         if stash not in self._auto_drop:
             if stash not in self._stashes:
-                self._stashes[stash] = list()
+                self._stashes[stash] = []
             self._stashes[stash].extend(states)
 
     def _clear_states(self, stash):
@@ -729,9 +779,9 @@ class SimulationManager:
     #   ...
     #
 
-    def _create_integral_stashes(self):
+    def _create_integral_stashes(self) -> DefaultDict[str, List['SimState']]:
         stashes = defaultdict(list)
-        stashes.update({name: list() for name in self._integral_stashes})
+        stashes.update({name: [] for name in self._integral_stashes})
         return stashes
 
     def _copy_stashes(self, deep=False):
@@ -802,9 +852,4 @@ class ErrorRecord:
     def __eq__(self, other):
         return self is other or self.state is other
 
-
-from .errors import SimError, SimMergeError
-from .sim_state import SimState
-from .state_hierarchy import StateHierarchy
-from .errors import AngrError, SimUnsatError, SimulationManagerError
 from .exploration_techniques import ExplorationTechnique, Veritesting, Threading, Explorer
